; FindRecoMapOffset
;
; this function finds the RECO_map_offset values
; the Bruker system applied to the image. The RECO_map_offset values are saved
; in the Bruker reco file i.e., scanName.ix3/scanNum/pData/1/reco
;
;	inputs:		directory	=> pointer (LUN) to the bruker 'acqp' file describing
;							   various parameters relating to an acquired image
;				scanNumString => string representing the bruker scan number
;
;	outputs:	recoMapOffset => value of map offset defined in reco file
;
; 	DGB 3/20/03
;
function FindRecoMapOffsetT2, directory
	recoFile = 3
	stringFound = 0			; indicate the RECO_map_offset string was found
	exitWhile = 0			; used to exit while loop
	recoFileName = directory+'/pData/1/reco'
	openr, recoFile, recoFileName
	line = ''
	while not eof(recoFile) and exitWhile NE 1 do begin	; search the reco file until RECO_map_offset string is found
		readf, recoFile, line
		if stringFound EQ 1 then begin		; found the string. Now prepare to retreive the next line
			exitWhile = 1
			mapOffsetLen = strpos(line,' ')	; length of string representing the first dimension of image
			if mapOffsetLen LT 0 then begin	; this will happen if only one value exists in offset field
				mapOffsetLen = strlen(line)	; i.e., one image was acquired in this scan
				mapOffset = float(strmid(line,0,mapOffsetLen))
				;print, mapOffsetLen
				;print, mapOffset
			endif else begin

				; now parse the line containing the multiple map offset values
				spacePosition = strpos(line,' ')
				while spacePosition GT 0 do begin
					mapOffset
					firstDimLen = strpos(line,' ')	; length of string representing the first dimension of image
					secondDimLen = strlen(line)		; second dimension part of string goes to end of entire string
					nFreqEncode = long(strmid(line,0,firstDimLen))	; convert freq encode string to int
					nPhaseEncode = long(strmid(line,firstDimLen + 1,secondDimLen-1)) ;convert phase encode string to int

				endwhile
			endelse

			;print, mapOffsetLen
			;break
			;mapOffsetLen = strlen(line)		; second dimension part of string goes to end of entire string
			;print, 'Found the String!!!!!!!!'
			;convert Real and Im part of DC Offset string to int
			;mapOffset = long(strmid(line,0,reOffsetLen))
			;imOffset = long(strmid(line,reOffsetLen + 1,imOffsetLen-1))
			;print, reOffset
			;print, imOffset
		endif
		if strpos(line,'RECO_map_offset=') GT 0 then stringFound = 1 	; if the ACQ_size string is found set the flag

	endwhile

close, recoFile
return, [mapOffset]

end

