; This program produces the T1 parameter map of images acquired using
; the bruker MSME sequence.
; DGB 4/17/03
;
; Created specifically for MQP project (LRC). 1/24/05
pro T1MapReco_MQP_goodversion_adjusted
	rotate90 = 1
	rotate180 = 2
	rotate270 = 3
	rotateTransOnly = 4
	rotateTrans90 = 5
	rotateTrans180 = 6
	rotateTrans270 = 7
	threshold = 0.15 	; for median smoothing of parameter maps
	plotNames = ['Signal Intensity: Scaling', 'Signal Intensity: No Scaling']
	sizeIndex = 3 ;# of elements in array is returned by 'size' as 4th element


	; ENTER the directory numbers where the images of interest are saved
	; i.e., scan number 4 used TR = 20 ms.
	;       scan number 5 used TR = 50 ms, etc...
	dirIndex = [3,4,5,6,7,8,9,10,11,12]	; image directories
 	TR = [0.18,0.2,0.23,0.3,0.55,0.95,1.65,3.15,5.15,7.65]	;TR values (T1 weight)
 	;TR = [3.0,1.5,0.80,0.40,0.15,0.08,0.05,0.03]

	outLyerFactor = 5.0	; This value is multiplied by the standard deviation of the map's rows.
						; This provides a threshold (or maximum) against which all
						; pixels in each row of the map are compared for outlyer removal.
	minOutLyers = 1

	nIndex = size(dirIndex)		; get size of index array
	nScans = nIndex(sizeIndex)	; # of directories to search through
	pixelsNoScale = fltarr(nIndex(sizeIndex))	; set up pixels array dynamically
	pixelsScale = fltarr(nIndex(sizeIndex))

	; ************************************
	; Mz and M0 as in: Mz(t) = M0(1-exp^-TR/T1)
	Mz = fltarr(nScans,256,256)
	M0 = fltarr(256,256)
	; ************************************

	noisePixels = fltarr(256,256)
	noisePerImage = fltarr(nScans)
	noisePixels[*,*] = 0

	directory = ''
	; done with intializations so prompt user for image directory
	READ, directory, PROMPT=$
	'Enter parent directory under which your raw data is stored: '

	directory = directory + '/'

	;t1outputfile=''
	;READ, t1outputfile, PROMPT=$
	;'What do you want for the filename: '

	close, /ALL

	; **************************************************************************************
	; Gather the reconstructed images from the Bruker directory.
	; then calculate statistics on the images to be used for T1 mapping
	; thresholds.
	for loopNum = 0,nScans - 1 do begin
		;strtrim,2 removes leading + trailing blanks
		scanNumString = strtrim(string(dirIndex(loopNum)),2)
		filename = directory+scanNumString+'/pdata/1/2dseq'
		matrixSize = FindMatrixSize(directory, scanNumString) ; get size of image
		nFreqEncode = matrixSize[0]/2
		nPhaseEncode = matrixSize[1]
		nImages = FindNumberImages(directory, scanNumString)  ; # images per scan
		dcOffset = FindDCOffset(directory, scanNumString)	  ; DC offset/image
		mapSlope = FindRecoMapSlope(directory, scanNumString)
		mapOffset = FindRecoMapOffset(directory, scanNumString)
		Mz[loopNum,*,*] = ProcessReco(filename, matrixSize, mapOffset, mapSlope)
		;now determine the average signal intensity (log value) of the center and
		;left corner of the image.
		;imageStats is an array of values holding the following statisitcs:
		;[centerMean, cornerMean, centerVar, cornerVar, centerSdv, cornerSdv]
		noiseFloor_MQP = imageseg_MQP(Mz[loopNum,*,*],nFreqEncode, nPhaseEncode)
		noisePerImage[loopNum] = noiseFloor_MQP

	endfor

	; **************************************************************************************

	;for k=0,nScans-1L do begin
	;	live_image, reform(Mz[k,*,*],256,256)
	;endfor

	; **************************************************************************
	; the y-axis of the linear fit curve is ln[Mz(t)] - ln[M0].
	; don't subtract the m0Image from itself (hence the nScans - 2).
	Y = fltarr(nScans,nFreqEncode,nPhaseEncode) ; as in y = mx+b => for linear fitting
	M0 = Mz[nScans-1,*,*] ; M0 is the image taken with longest TR (3000 ms)
	LogM0 = alog(M0)	  ; natural logarithm of M0
	LogMz = alog(Mz)
	;print, size(LogM0)


	for is = 0,nScans - 2 do begin
		Y[is+1,*,*] = LogMz[is,*,*] - LogM0[0,*,*]	;ECH
		;Y[is,*,*] = LogM0[0,*,*]-LogMz[is,*,*]
	endfor

	Y[0,*,*]= LogM0	;ECH
	;Y[nScans-1,*,*] = LogM0 ; last data point on y axis is approximation to M0

	for k=0,nScans-1L do begin	;ECH
		live_image, reform(Mz[k,*,*],256,256)	;ECH
	endfor	;ECH

	; **************************************************************************


	;logIntensities = alog(imagesToFit) ; nImages x 256 x 128 array

	t1MapLin = fltarr(nFreqencode,nPhaseEncode)
	t1MapLinNoOutLyers = fltarr(nFreqencode,nPhaseEncode)
	m0PrimeMapLin = fltarr(nFreqEncode,nPhaseEncode)
	t1MapNonLin = fltarr(nFreqencode,nPhaseEncode)
	t1MapNonLinNoOutLyers = fltarr(nFreqencode,nPhaseEncode)
	m0PrimeMapNonLin = fltarr(nFreqEncode,nPhaseEncode)

	; ***************************************
	; Do LINEAR curve fitting of T1
	; ***************************************
	for i = 0,nFreqEncode - 1 do begin
		for j = 0,nPhaseEncode - 1	do begin
			;check that current pixel in EACH (*) image has higher value
			;than noise floor of each (*) image. GT will return boolean
			;vector. total will sum boolean vector. If each image's pixel is GT
			;the noise floor value it is most likely a good pixel to fit.
			if fix(total(Mz[*,i,j] GT noisePerImage[*])) EQ nScans then begin
					model = linfit(TR,Y[*,i,j])
				;if model[1] GT 0 then begin
				if model[1] LT 0 then begin	;ECH
					t1MapLin[i,j] = 1.0/(-model[1])
				endif
					m0PrimeMapLin[i,j] = exp(model[0])
			endif else begin
				noisePixels[i,j] = 1
			endelse
		endfor
	endfor

	t1MapLinNoOutLyers = t1MapLin

	;RemoveOutLyers, t1MapLinNoOutLyers,nFreqEncode,nPhaseEncode,outLyerFactor,minOutLyers

	; *************************************************************
	; Do NON_LINEAR curve fitting for T1 using the curvefit routine
	; *************************************************************
	;initGuess = fltarr(2)
	;A = fltarr(2)
	;yfit = fltarr(3)
	; provide initial guess of non-linear function parameters
	;initGuess = [mean(t1MapLin),1.0]
	;W = fltarr(N_ELEMENTS(Mz[*,1,1]))
	;W(*) = 1.0
	;A = initGuess

	;for i = 0,nFreqEncode - 1 do begin
	;	for j = 0,nPhaseEncode - 1	do begin
	;		;check that current pixel in EACH (*) image has lower value
	;		;than corner average of each (*) image. LT will return boolean
	;		;vector. total will sum boolean vector. if each image's pixel is LT
	;		;the cornerMean value it is most likely a noise pixel, so don't do the
	;		;T2 fit for that pixel.
	;		if fix(total(Mz[*,i,j] GT noisePerImage[*])) EQ nScans then begin
	;			yfit = Curvefit(TR,Y(*,i,j),W,A,SIGMA,FUNCTION_NAME = 'T1curveFitFunc')
	;			t1MapNonLin[i,j] = A[1]
	;			m0PrimeMapNonLin[i,j] = A[0]
	;		endif else begin
	;			noisePixels[i,j] = 1
	;		endelse
	;	endfor
	;endfor

	;t1MapNonLinNoOutLyers = t1MapNonLin

	;RemoveOutLyers, t1MapNonLinNoOutLyers,nFreqEncode,nPhaseEncode,outLyerFactor,minOutLyers

	index=where(m0PrimeMapLin GT 4.0)
	m0PrimeMapLin[index]=0.0
	t1MapLin[index]=0.0
	;t1MapNonLin[index]=0.0
	;m0PrimeMapNonLin[index]=0.0

	;Change scaling so that smaller contrasts in T1 and T2 register different values in color scheme

	;index=where(m0PrimeMapLin LT 1.2)
	;m0PrimeMapLin[index]=1.2
	;index=where(m0PrimeMapLin GT 4.0)
	;m0PrimeMapLin[index]=4.0
	;index=where(t1MapLin LT 1.1)
	;t1MapLin[index]=1.1
	;index=where(t1MapLin GT 2.0)
	;t1MapLin[index]=2.0

	live_image, rotate(noisePixels, rotateTrans270), TITLE = 'Noise Pixels-ECH'
	live_image, rotate(t1MapLin, rotateTrans270), TITLE = 'Lin Fit T1-ECH'
	live_image, rotate(m0PrimeMapLin, rotateTrans270), TITLE = 'Lin Fit M0-ECH''
	;live_image, rotate(t1MapNonLin, rotateTrans270), TITLE = 'Non-Lin Fit T1'
	;live_image, rotate(m0PrimeMapNonLin, rotateTrans270), TITLE = 'Non-Lin Fit M0''
	;m0PrimeMapSmooth = SmoothMap(m0PrimeMap, threshold)
	;t1MapSmooth = SmoothMap(t1Map, threshold)
	;live_image, rotate(t1MapSmooth, rotateTrans270)
	;live_image, rotate(m0PrimeMapSmooth, rotateTrans270)

	;close,1
	;openw,1,"g:\temp\"+t1outputfile
	;writeu,1, t1maplin
	;close,1

end