; FindDCOffset
;
; this function finds the DC offset values (applied to RE and IM data)
; the Bruker system applied to the image. The DC offset values are saved
; in the Bruker reco file i.e., scanName.ix3/scanNum/pData/1/reco
;
;	inputs:		directory	=> pointer (LUN) to the bruker 'acqp' file describing
;							   various parameters relating to an acquired image
;				scanNumString => string representing the bruker scan number
;
;	outputs:	reOffset => DC offset applied to Real image data
;							(as determined from Bruker reco file).
;
;				imOffset => DC offset applied to Imaginary image data
;
; 	DGB 3/20/03
;
function FindDCOffsetT2, directory
	recoFile = 3
	stringFound = 0			; indicate the RECO_dc_offset string was found
	exitWhile = 0			; used to exit while loop
	recoFileName = directory+'\pData\1\reco'
	openr, recoFile, recoFileName
	line = ''
	while not eof(recoFile) and exitWhile NE 1 do begin	; search the reco file until RECO_dc_offset string is found
		readf, recoFile, line
		if stringFound EQ 1 then begin		; found the string. Now prepare to retreive the next line
			exitWhile = 1
			reOffsetLen = strpos(line,' ')	; length of string representing the first dimension of image
			imOffsetLen = strlen(line)		; second dimension part of string goes to end of entire string
			;print, 'Found the String!!!!!!!!'
			;convert Real and Im part of DC Offset string to int
			reOffset = long(strmid(line,0,reOffsetLen))
			imOffset = long(strmid(line,reOffsetLen + 1,imOffsetLen-1))
			;print, reOffset
			;print, imOffset
		endif
		if strpos(line,'RECO_dc_offset=') GT 0 then stringFound = 1 	; if the ACQ_size string is found set the flag

	endwhile

close, recoFile

;print, reOffset
;print, imOffset

return, [reOffset, imOffset]

end

