; FindNumberImages
;
; this function finds the number of images (slices) acquired in a scan as saved
; in the Bruker acqp file.
;
;
;	inputs:		directory	=> pointer (LUN) to the bruker 'acqp' file describing
;							   various parameters relating to an acquired image
;				scanNumString => string representing the bruker scan number
;
;	outputs:	nImages => # of images (slices) acquired in one scan
;							(as determined from Bruker acqp file).
;
; 	DGB 3/18/03
;	Modified by Stuart Howes 4/23/05

function FindNumberImagesT2, directory
	acqpFile = 2
	niStringFlag = 0	; ##$NI=n line of acqp file refers to number of images (slices) acquired
	acqpFileName = directory+'\acqp'
	openr, acqpFile, acqpFileName, /SWAP_ENDIAN, GET_LUN
	line = '' & READF, acqpFile, line

	while not eof(acqpFile) and niStringFlag EQ 0 do begin	; search the acqp file until
															; ACQ_size string is found
		readf, acqpFile, line
		if strpos(line,'$NI=') GT 0 then begin
			niStringFlag = 1 				; if the ACQ_size string is found set the flag
			numberPosition2 = strlen(line) - 1	; numbef of images 'number' is at end of string
			numberPosition1 = strpos(line,'=') + 1	; be ready for 2 digits in number
													; (i.e., 10 images vs. 8 images)
			if numberPosition2 GT numberPosition1 then begin
				tens = fix(strmid(line,numberPosition1))
				ones = fix(strmid(line,numberPosition2))
				nImages = fix(tens + ones)
			endif else begin
				nImages = fix(strmid(line, numberPosition1))
			endelse
		endif
	endwhile

close, acqpFile
return, nImages

end
