; T2 Map Program
; Created by Stuart Howes for Collagen Sponge Degradation MQP
; 4/23/05
; First attempt to program from scratch

pro T2mapping

;Definitions

directory = ''
sizeIndex = 3
TE = [0.02, 0.04, 0.06, 0.08, 0.1, 0.12, 0.14, 0.16, 0.18, 0.2, 0.22, 0.24]
nIndex = size(TE)
nEch = nIndex(sizeIndex)
rotatetrans180 = 2
rotatetrans270 = 3


;Prompt user for directory where data is stored
READ, directory, PROMPT=$
	'Enter scan directory under which your raw T2 data is stored.'
directory = directory + '\'
close, /ALL
filename = directory+'\pdata\1\2dseq'

;Collect data acquisition parameters
matrixSize = FindMatrixSizeT2(directory)
nFreqEncode = matrixSize[0]
nPhaseEncode = matrixSize[1]
dcOffset = FindDCOffsetT2(directory)
;mapslope = 0.0157687
mapSlope = 1.07893E-05
mapOffset = 0

noScaleImage = lonarr(nEch,nFreqEncode,nPhaseEncode)
brukerImage = intarr(nFreqEncode,nEch*nPhaseEncode)
hanningWindow = hanning(nFreqEncode, nPhaseEncode, ALPHA=.50)
openr, 1, filename, /swap_endian
readu, 1, brukerImage

live_image, brukerImage, title='Raw Bruker Image'

for h=0,nEch-1	do begin
	for j=0, nPhaseEncode - 1 do begin
		for i=0, (nFreqEncode) - 1 do begin
			noScaleImage[h,i,j] = mapOffset + ((1/mapSlope)*brukerImage[i,j+h*256])
		endfor
	endfor
	;live_image, rotate((reform(noScaleImage[h,*,*],nFreqEncode,nPhaseEncode)),rotatetrans180), title='Raw Image, echo number'+string(TE[h])
endfor

close, 1

;**********************************************************************************
;*					      Make and Apply the Mask								  *
;**********************************************************************************

MASK = fltarr(nFreqEncode, nPhaseEncode)

for i=0,nFreqEncode-1 do begin
	for j=0,nPhaseEncode-1 do begin
		MASK[i,j]=noScaleImage[0,i,j]
	endfor
endfor

live_image, rotate(MASK,2), title='Image from which Mask is made, shortest TE.'

;Initial Masking
for i = 0,nFreqEncode-1 do begin
	for j = 0,nPhaseEncode-1 do begin
		if MASK[i,j] LT 190000 then begin
		  MASK[i,j] = 0
		endif else begin
		  MASK[i,j] = 1
		endelse
	endfor
endfor

;live_image, rotate(MASK,2), title='Mask without checking surrounding pixels'

;Make sure that the pixels you are eleminating are in regions outside the sponges
;i.e. not surrounded by 1s in the MASK
for i = 10,nFreqEncode-10 do begin
	for j = 10,nPhaseEncode-10 do begin
	  if MASK[i,j] EQ 0 Then Begin
	   if MASK[i-2,j-2] EQ 1 AND MASK[i-1,j-2] EQ 1 AND MASK[i,j-2] EQ 1 AND MASK[i+1,j-2] EQ 1 AND MASK[i+2,j-2] EQ 1 AND $
	  	 MASK[i-2,j-1] EQ 1 AND MASK[i-1,j-1] EQ 1 AND MASK[i,j-1] EQ 1 AND MASK[i+1,j-1] EQ 1 AND MASK[i+2,j-1] EQ 1 AND $
	     MASK[i-2,j] EQ 1 AND MASK[i-1,j] EQ 1 AND MASK[i+1,j] EQ 1 AND MASK[i+2,j] EQ 1 AND $
	     MASK[i-2,j+1] EQ 1 AND MASK[i-1,j+1] EQ 1 AND MASK[i,j+1] EQ 1 AND MASK[i+1,j+1] EQ 1 AND MASK[i+2,j+1] EQ 1 AND $
	     MASK[i-2,j+2] EQ 1 AND MASK[i-1,j+2] EQ 1 AND MASK[i,j+2] EQ 1 AND MASK[i+1,j+2] EQ 1 AND MASK[i+2,j+2] EQ 1 then begin
		MASK[i,j] =1
	  endif
	 endif
	endfor
endfor

live_image, rotate(MASK,2), title='Mask with surrounding pixels checked'

;Apply the Mask
MxyMasked = fltarr(nEch,nFreqEncode,nPhaseEncode)
for h=0,nEch-1 do begin
	for	i=0,nFreqEncode-1 do begin
		for j=0, nPhaseEncode-1 do begin
		MxyMasked[h,i,j] = noScaleImage[h,i,j]*MASK[i,j]
		endfor
	endfor
	;live_image, rotate((reform(noScaleImage[h,*,*],nFreqEncode,nPhaseEncode)),2), title='Masked Image, echo number'+string(TE[h])
endfor

M0Masked = fltarr(nFreqEncode,nPhaseEncode)

for i=0,nFreqEncode-1 do begin
	for j=0,nPhaseEncode-1 do begin
		M0Masked[i,j]=noScaleImage[0,i,j]
	endfor
endfor

Y = fltarr(nEch,nFreqEncode,nPhaseEncode)
for is=0,nEch-1 do begin
	Y[is,*,*]=alog(MxyMasked[is,*,*]);  - alog(M0Masked)
endfor
;Y[0,*,*]=M0Masked

t2MapLin = fltarr(nFreqEncode,nPhaseEncode)
M0MapLin =fltarr(nFreqEncode,nPhaseEncode)

for i=0,nFreqEncode-1 do begin
	for j=0,nPhaseEncode-1 do begin
		model=linfit(TE,Y[*,i,j])
		if model[1] LT 0 then begin
			t2MapLin[i,j]=1/(-model[1])
		endif
	endfor
endfor

;Read in half the image
t2mapLinH = fltarr(nFreqEncode/2,nPhaseEncode)
for i=0,nFreqEncode/2-1 do begin
	for j=0,nPhaseEncode-1 do begin
		t2MapLinH[i,j]=t2Maplin[i,j]
	endfor
endfor

live_image, rotate(t2MapLin,2), title = 'Lin Fit T2 - Not Scaled'
;live_image, rotate(t2MapLinH,2), title = 'Lin Fit T2 - Not Scaled'

	index=where(t2MapLin GT 0.7)
	s=size(index)
if s[0] GT 0 then begin
	t2MapLin[index]=0
endif;  else begin
	;t2MapLin[0,0] = 2.6e-6
;endelse

;	index=where(t2MapLin LT 1.5e-7)
;	s=size(index)
;if s[0] GT 0 then begin
;	t2MapLin[index]=1.5e-7
;endif else begin
;	t2MapLin[0,1] = 1.5e-7
;endelse

;	index=where(t2MapLinH GT 2.6e-6)
;	s=size(index)
;if s[0] GT 0 then begin
;	t2MapLinH[index]=2.6e-6
;endif else begin
;	t2MapLinH[0,0] = 2.6e-6
;endelse

;	index=where(t2MapLinH LT 1.5e-7)
;	s=size(index)
;if s[0] GT 0 then begin
;	t2MapLinH[index]=1.5e-7
;endif else begin
;	t2MapLinH[0,1] = 1.5e-7
;endelse

live_image, rotate(t2MapLin, 2), title = 'Lin Fit T2 - Scaled'
;live_image, rotate(t2MapLinH, 2), title = 'Lin Fit T2 - Scaled'

end
