; FindMatrixSize
;
; this function finds the image matrix size as saved in the Bruker acqp file.
; NOTE that the acqp file stores the real and imaginary frequency components
; so this number needs to be divided by two.
;
;
;	inputs:		acqpFile	=> pointer (LUN) to the bruker 'acqp' file describing various parameters
;							   relating to an acquired image
;				scanNumString => string representing the bruker scan number
;
;	outputs:	nFreqEncodes => # of frequency encoding steps used to generate the image.
;
;				nPhaseEncodes => # of phase encoding steps used to generate the image.
;
; Modified by Stuart Howes 4/23/05

function FindMatrixSizeT2, directory
	fidFile = 1
	acqpFile = 2
	stringFound = 0
	exitWhile = 0
	acqpFileName = directory+'\pdata\1\reco'

	; DEBUG
	;print, acqpFileName
	;STOP_HERE
	;DEBUG

	openr, acqpFile, acqpFileName
	line = '' & READF, acqpFile, line
	while not eof(acqpFile) and exitWhile NE 1 do begin	; search the acqp file until ACQ_size string is found
		readf, acqpFile, line
											; line contains the matrix size when acqpSizeFlag is set
		if stringFound EQ 1 then begin		; found the string. Now prepare to retreive the next line
			exitWhile = 1					; since it contains the data we need 'nFreqEncodes*2 nPhaseEncodes'
			firstDimLen = strpos(line,' ')	; length of string representing the first dimension of image
			secondDimLen = strlen(line)		; second dimension part of string goes to end of entire string
			nFreqEncode = long(strmid(line,0,firstDimLen))	; convert freq encode string to int
			nPhaseEncode = long(strmid(line,firstDimLen + 1,secondDimLen-1)) ;convert phase encode string to int
		endif
		if strpos(line,'RECO_size=') GT 0 then stringFound = 1 	; if the ACQ_size string is found set the flag

	endwhile

;FREE_LUN, acqpFile
close, acqpFile
FREE_LUN, fidFile
return, [nFreqEncode, nPhaseEncode]

end
