; FindRecoMapSlope
;
; this function finds the RECO_map_slope values
; the Bruker system applied to the image. The RECO_map_slope values are saved
; in the Bruker reco file i.e., scanName.ix3/scanNum/pData/1/reco
;
;	inputs:		directory	=> pointer (LUN) to the bruker 'acqp' file describing
;							   various parameters relating to an acquired image
;				scanNumString => string representing the bruker scan number
;
;	outputs:	recoMapSlope => value of map offset defined in reco file
;
; 	DGB 3/20/03
;
function FindRecoMapSlopeT2, directory
	recoFile = 3
	stringFound = 0			; indicate the RECO_map_offset string was found
	exitWhile = 0			; used to exit while loop
	recoFileName = directory+'\pData\1\reco'
	openr, recoFile, recoFileName
	line = ''
	while not eof(recoFile) and exitWhile NE 1 do begin	; search the reco file until RECO_map_offset string is found

		;DEBUG
		;print, 'In While loop of FindRecoMapSlope function'
		;STOP_HERE
		;DEBUG

		readf, recoFile, line
		if stringFound EQ 1 then begin		; found the string. Now prepare to retreive the next line

			; DEBUG
			;print, 'In first if statement in FindRecoMapSlope'
			;STOP_HERE
			; DEBUG

			exitWhile = 1
			mapSlopeLen = strpos(line,' ')	; length of string representing the first dimension of image

			; DEBUG
			print, 'mapSlopeLen is'
			print, mapSlopeLen
			;DEBUG

			if mapSlopeLen LT 0 then begin	; this will happen if only one value exists in offset field
				mapSlopeLen = strlen(line)	; i.e., one image was acquired in this scan
				mapSlope = float(strmid(line,0,mapSlopeLen))

				; DEBUG
				print, 'In second if statement in FindRecoMapSlope function'
				print, 'map slope is'
				print, mapSlope
				;STOP_HERE
				; DEBUG

			endif else begin

				; now parse the line containing the multiple map offset values
				spacePosition = strpos(line,' ')
				while spacePosition GT 0 do begin
					;mapOffset
					firstDimLen = strpos(line,' ')	; length of string representing the first dimension of image
					secondDimLen = strlen(line)		; second dimension part of string goes to end of entire string
					nFreqEncode = long(strmid(line,0,firstDimLen))	; convert freq encode string to int;
					nPhaseEncode = long(strmid(line,firstDimLen + 1,secondDimLen-1)) ;convert phase encode string to int
				endwhile
			endelse

		endif
		if strpos(line,'RECO_map_slope=') GT 0 then stringFound = 1 	; if the ACQ_size string is found set the flag

	endwhile

close, recoFile

print, mapSlope

;return, [mapSlope]

end

