; T2curveFitFunc
;
; This procedure is used in conjunction with IDL's curvefit function
; for fitting the non-linear model of signal intensity vs. TE time.
;
; note that func and pder are the RETURN variables.
;
;	Inputs: x 	=>  A row vector of independent variables from curvefit
;					parameter list. The curvefit routine does not manipulate
;					or use values in X, it simply passes X to this function
;					(function used in the KEYWORD FUNCTION_NAME)
;
;      	 	a	=> 	A vector from curvefit parameter list., with as many
;					elements as the number of terms, that contains the initial
;					estimate for each parameter.  IF A is double-precision,
;					calculations are performed in double precision, otherwise
;					they are performed in single precision. Fitted parameters
;           		are returned in A. Vector of NTERMS function parameters, input.
;       	func => Vector of NPOINT values of function, y(i) = funct(x), output.
;
;       	pder => Array, (NPOINT, NTERMS), of partial derivatives of funct.
;               	PDER(I,J) = Derivative of function at ith point with
;               	respect to jth parameter.  Optional output parameter.
;               	PDER should not be calculated IF the parameter is not
;               	supplied in call. IF the /NODERIVATIVE keyword is set in the
;               	call to CURVEFIT THEN the user routine will never need to
;               	calculate PDER.


pro T2curveFitFunc, X, A, func, pder

 	mZero = A[0]
 	T2 = A[1]
 	func = mZero * exp(-X/T2)

if N_PARAMS() GE 4 then $
	pder = [[exp(-X/T2)], $ 				  ; dM/dM0
    	  	[mZero * (X/(T2^2)) * exp(-X/T2)]]; dM/dT2
end


