; T1Funct
;
; This procedure is used in conjunction with IDL's curvefit function
; for fitting the non-linear model of T1 relaxation vs. TR time.
;
; note that f and pder are RETURN values!!!
;
;	Inputs: x 	=>  A row vector of independent variables from curvefit
;					parameter list. The curvefit routine does not manipulate
;					or use values in X, it simply passes X to this function
;					(function used in the KEYWORD FUNCTION_NAME)
;
;      	 	a	=> 	A vector from curvefit parameter list., with as many
;					elements as the number of terms, that contains the initial
;					estimate for each parameter.  IF A is double-precision,
;					calculations are performed in double precision, otherwise
;					they are performed in single precision. Fitted parameters
;           		are returned in A. Vector of NTERMS function parameters, input.
;       	f 	=> 	Vector of NPOINT values of function, y(i) = funct(x), output.
;
;       	pder => Array, (NPOINT, NTERMS), of partial derivatives of funct.
;               	PDER(I,J) = Derivative of function at ith point with
;               	respect to jth parameter.  Optional output parameter.
;               	PDER should not be calculated IF the parameter is not
;               	supplied in call. IF the /NODERIVATIVE keyword is set in the
;               	call to CURVEFIT THEN the user routine will never need to
;               	calculate PDER.
;
; T1model[0] = a, T1model[1] = b, T1model[2] = c of model:
; F(t) = a - a*exp(-bt) + c

pro T1curveFitFunc, t, T1model, func, pder

	;print, 'in T1Funct'
	;print, N_PARAMS()

 temp = exp(-T1model[1] * t)
 func = T1model[0] - T1model[0]*temp + T1model[2] ; M0 - M0exp(-t/T1) + c (0)

;If the procedure is called with four parameters, calculate the
;partial derivatives.
 if N_PARAMS() GE 4 then $
 	pder = [[1 - temp], [T1model[0] * t * temp], [replicate(1.0, N_ELEMENTS(func))]]

 end
