; T1LMFitfunc

function T1LMFitfunc,X,A
;
;Return a vector appropriate for LMFIT
;
;         The function being fit is of the following form:
;           F(x) = A[0] * ( 1 - A[1]exp( -X/A[2]))
;         	M = M0(1 - alpha * exp(-TR/T1)

;         dF/dA(0) is dM/dA(0) = 1 - A[1]*exp(-X/A[2])
;         dF/dA(1) is dM/dA(1) = -A[0]*exp(-X/A[2])
;         dF/dA(2) is dM/dA(2) = A[0]*A[1]*(X/A[2]^2)*exp(-X/A[2])
;
;         return,[[F(x)],[dF/dA(0)],[dF/dA(1)],[dF/dA(2)]]
;
;         Note: returning the required function in this manner
;             ensures that if X is double the returned vector
;             is also of type double. Other methods, such as
;             evaluating size(x) are also valid.
;
		mZero = A[0]
		effFactor = A[1] ; or alpha
		T1 = A[2]

		B = [ [mZero*(1-effFactor*exp(-X/T1))], $; function
        	[1 - effFactor * exp(-X/T1)], $ 			; dM/dM0
        	[-mZero * exp(-X/T1)], $  					; dM/dalpha
        	[-mZero * effFactor * (X/T1^2) * exp(-X/T1)]]; dM/dT1

		print, 'T1LMFitFunc: here are the factors'
        print, X
        print, effFactor
        print, T1
        print, B

        return, B




	end