; T1curveFitFunc
;
; This procedure is used in conjunction with IDL's curvefit function
; for fitting the non-linear model of signal intensity vs. TR time.
;
; note that func and pder are RETURN values!!!
;
;	Inputs: x 	=>  A row vector of independent variables from curvefit
;					parameter list. The curvefit routine does not manipulate
;					or use values in X, it simply passes X to this function
;					(function used in the KEYWORD FUNCTION_NAME)
;
;      	 	a	=> 	A vector from curvefit parameter list., with as many
;					elements as the number of terms, that contains the initial
;					estimate for each parameter.  IF A is double-precision,
;					calculations are performed in double precision, otherwise
;					they are performed in single precision. Fitted parameters
;           		are returned in A. Vector of NTERMS function parameters, input.
;       	func => Vector of NPOINT values of function, y(i) = funct(x), output.
;
;       	pder => Array, (NPOINT, NTERMS), of partial derivatives of funct.
;               	PDER(I,J) = Derivative of function at ith point with
;               	respect to jth parameter.  Optional output parameter.
;               	PDER should not be calculated IF the parameter is not
;               	supplied in call. IF the /NODERIVATIVE keyword is set in the
;               	call to CURVEFIT THEN the user routine will never need to
;               	calculate PDER.
;
; T1model[0] = a, T1model[1] = b, T1model[2] = c of model:
; F(t) = a - a*exp(-bt) + c

pro T1curveFitFunc, X, A, func, pder

	if N_ELEMENTS(A) EQ 3 then begin
		mZero = A[0]
		effFactor = A[1] ; or alpha
		T1 = A[2]
		func = mZero*(1-effFactor*exp(-X/T1))
	endif else begin
 		if N_ELEMENTS(A) EQ 2 then begin
 			mZero = A[0]
 			T1 = A[1]
 			func = mZero*(1 -  2 * exp(-X/T1))
 		endif
	endelse

	; use the following when the efficiency factor is included in model
	if N_ELEMENTS(A) EQ 3 then begin
		if N_PARAMS() GE 4 then begin
			pder = [[1 - effFactor * exp(-X/T1)], $ 			; dM/dM0
    	   			[-mZero * exp(-X/T1)], $  					; dM/dalpha
    	   			[-mZero * effFactor * (X/(T1^2)) * exp(-X/T1)]]; dM/dT1endif
		endif
	endif else begin
	; use when efficiency factor is NOT included in model
		if N_ELEMENTS(A) EQ 2 then begin
			if N_PARAMS() GE 4 then begin
				pder = [[1 - 2*exp(-X/T1)], $ 			; dM/dM0
    	   		[-mZero * 2 * (X/(T1^2)) * exp(-X/T1)]]	; dM/dT1
			endif
		endif
	endelse

end ; end of T1curveFitFunc


