; T2LMFitfunc2Param

function T2LMFitfunc2Param,X,A
;
;Return a vector appropriate for LMFIT. 2 parameter fit for T2
;
;         The function being fit is of the following form:
;           F(x) = A[0] * exp( -X/A[1]))
;         	M = M0 * exp(-TE/T2)

;         dF/dA[0] is dM/dA[0] = exp(-X/A[1])
;         dF/dA[1] is dM/dA[1] = A[0]*(X/A[1]^2)*exp(-X/A[1])
;
;         return,[[F(x)],[dF/dA(0)],[dF/dA(1)]]
;
;         Note: returning the required function in this manner
;             ensures that if X is double the returned vector
;             is also of type double. Other methods, such as
;             evaluating size(x) are also valid.
;
		mZero = A[0]
		T2 = A[1]

		B = [ [mZero * exp(-X/T2)], $				; function
			  [exp(-X/T2)], $ 						; dM/dM0
        	  [mZero * (X/T2^2) * exp(-X/T2)]]		; dM/dT2
        return, B

	end