; function ImageSegment
;
; this function segments an image according to the average pixel values
; of the center of the image (60x60) and the left corner of the image (30x30).
; It is assumed that the image is centered and noise is randomly distributed
; throughout the outer 'edge' of the image.
;
;	Input: imageStats		=> structure to fill regarding various stats of image mask
;		   sigIntensities 	=> matrix holding log of signal intensity values of an image
;		   nFreqEncode 		=> # of frequency encoding steps used to acquire image(s)
;		   nPhaseEncode 	=> # of phase encoding steps used to acquire image(s)
;		   nScans			=> # of scans taken (= # of directories under scan name)
;		   boxWidth			=> # of pixels in freq encode dir used to create image mask or
;								image segment box
;		   boxHeight		=> # of pixels in phase encode dir used to create image mask or
;								image segment box
;
;	Output:
;			centerMean	=>	mean of image center pixels (boxPixels x boxPixels)
;			cornerMean	=>  mean of image left corner pixels (boxPixels/2 x boxPixels/2)
;			centerVAR	=>	variance of image center pixels
;			cornerVAR 	=> 	variance of image left corner pixels
;			centerSDV 	=>	standard deviation of image center pixels
;			cornerSDV	=>	standard deviation of image left corner pixels
;
;		*       nFreqEncode           *
;		----------------------------- *
;		|...	    				| n
;		|. . <- 	sample			| P
;		|..	 crnr   		   		| h
;		|. .   		sample			| a
;		|. .    			    	| s
;		|. .        sample          | E
;		|. .                        | n
;		|. .		sample			| c
;		|. .						| o
;		|...						| d
;		----------------------------- *
;
;
;	History: 1/24/05: Specifically designed for MQP project images :
; 					Laura's group

function imageseg_MQP, sigIntensities, nFreqEncode, nPhaseEncode


; DEBUG
print, 'nFreqEncode in ImageSegment is:'
print, nFreqEncode
print, 'nPhaseEncode in ImageSegment is:'
print, nPhaseEncode
print, ''
print, 'size of sigIntensities is:'
print, size(sigIntensities)
print, ''
print, ''
; DEBUG

; JUST make the corner a rectangle to cover almost the whole
; side of the image...only want to avoid the center of the image!
;sideImage = intarr(nFreqEncode,nPhaseEncode)
;sideImage[0:nFreqEncode - 176,*] = 1
;sideImage[nFreqEncode-82:255,*] = 1
; DEBUG
;live_image, sideImage
;DAVE
; DEBUG

leftSide = sigIntensities[0,nFreqEncode - 180,*]
rightSide = sigIntensities[0,nFreqEncode-60:255,*]

image_statistics, leftSide, MEAN=leftSide_Mean, VARIANCE=leftSide_Var, $
				 STDDEV=leftSide_Sdv
image_statistics, rightSide, MEAN=rightSide_Mean, VARIANCE=rightSide_Var, $
				 STDDEV=rightSide_Sdv
noiseFloor_MQP = mean(leftSide_Mean+rightSide_Mean)
return, noiseFloor_MQP

end
