; ProcessReco
;
; input:
;		 filename	=> path of file holding bruker 2dseq file
;		 matrixSize	=> array holding # of columns (frequency encodes, RE and IM) and
;					   # of rows (phase encodes) used to acquire image
;		 mapOffset	=> bruker reco variable. Offset applied to reconstructed image?
;		 mapSlope	=> bruker reco variable. Constant applied to reconstructed image?
;
; output: noScaleImage => unscaled (bruker scaling removed) reconstructed bruker image
;
function ProcessRecoH, filename, matrixSize, mapOffset, mapSlope
	seqFile = 1
	rotate90 = 1
	rotate180 = 2
	rotate270 = 3
	rotateTransOnly = 4
	rotateTrans90 = 5
	rotateTrans180 = 6
	rotateTrans270 = 7
	nFreqEncode = 256	; remember nFreqEncode includes RE and IM parts
	nPhaseEncode = 128
	noScaleImage = lonarr(nFreqEncode,nPhaseEncode)
	brukerImage = intarr(nFreqEncode,nPhaseEncode)	; images are 16-bit signed int
	hanningWindow = hanning(nFreqEncode, nPhaseEncode, ALPHA=.50)
	openr, seqFile, filename, /swap_endian
	readu, seqFile, brukerImage	; read unformatted binary data into variable
								;image from opened file

	for i=0L, nFreqEncode - 1 do begin
		for j=0L, nPhaseEncode - 1 do begin
			noScaleImage[i,j] = mapOffset + ((1/mapSlope)*brukerImage[i,j])
		endfor
	endfor

	noScaleImage = noScaleImage * hanningWindow

	;live_image, rotate(noScaleImage, rotate180)
	;live_image, rotate(noScaleImage, rotateTrans270)
	;live_surface, rotate(noScaleImage, rotateTrans270)

	;pixelsNoScale[loopNum] = noScaleImage[100,90]
	;pixelsScale[loopNum] = brukerImage[100,90]

	close, seqFile
	return, noScaleImage

end
