; SmoothMap
; used to smooth (average out pixels in 3x3 kernel area)
; the m0 parameter map
;
; Inputs:
;	map			=> map of M0
;	threshold 	=> average of kernel pixels are compared to current pixel.
;				   Comparison value is threshold value.
;
; Outputs:
;	moMapSmooth	=> the 'smoothed' version of the moMap input
;
;	DGB 5/5/03
;
function SmoothMap, map, threshold

	imageKernel = fltarr(3,3)
	kernel = intarr(3,3)
	kernel(*,*) = 1
	;kernel(1,1) = 0	; used for Out-Range Pixel Smoothing
	sizeMap = size(map)
	sizeKernel = size(kernel)
	colsMap = sizeMap(1)
	rowsMap = sizeMap(2)
	nKernelElements = sizeKernel(1) * sizeKernel(2)
	;print, total(kernel)

	; skip 0, 255 columns and rows since 3x3 kernel doesn't
	; cover these rows/columns symmetrically
	; to avoid skipping columns/rows 0 & 255 use separable median
	; filtering
	for js = 1,colsMap - 2 do begin
		for is = 1,rowsMap - 2 do begin
			imageKernel = map[js-1:js+1,is-1:is+1] ; 3x3 section of current image
			imageKernel = imageKernel * kernel
			average = total(imageKernel)/nKernelElements
			; replace the current pixel value with the average of surrounding
			; values (1 pixel surround) if the difference between the average
			; and the current pixel value is greater than a specific threshold
			; try using MEDIAN?
			if abs(map(js,is) - average) GT threshold then begin
				map(js,is) = average
			endif

			;print, imageKernel
			;print, average
		endfor
	endfor

	return, map
end

