; This program produces the T2 parameter map of images acquired using
; the bruker MSME sequence.
; DGB 3/12/03

pro t1t2MapReco
	rotate90 = 1
	rotate180 = 2
	rotate270 = 3
	rotateTransOnly = 4
	rotateTrans90 = 5
	rotateTrans180 = 6
	rotateTrans270 = 7
	plotNames = ['Signal Intensity: Scaling', 'Signal Intensity: No Scaling']
	sizeIndex = 3 ;# of elements in array is returned by 'size' as 4th element
	dirIndex = [4,5,6,7,8,9,10]		; image directories
	TE = [.03,.05,.07,.09,.120,.150,.200]	; TE values (T2 weight)
 	TR = [1.0,.70,.50,.30,.15,.08,.05,.028]	;TR values (T1 weight)
	nIndex = size(dirIndex)		; get size of index array
	nScans = nIndex(sizeIndex)	; # of directories to search through
	pixelsNoScale = fltarr(nIndex(sizeIndex))	; set up pixels array dynamically
	pixelsScale = fltarr(nIndex(sizeIndex))
	recoImages = fltarr(nScans,256,256)
	boxPixels = 60	; # of pixels used for averaging box=> segment image
	centerMean = fltarr(nScans)	;hold boxPixels x boxPixels part of image center
	centerMeanImage = fltarr(nScans,256,256)
	cornerMean = fltarr(nScans)	;"				" 		"      image left corner
	centerVar = fltarr(nScans)
	cornerVar = fltarr(nScans)
	centerSdv = fltarr(nScans)
	cornerSdv = fltarr(nScans)
	logIntensities = fltarr(nScans,256,256)
	noNoiseImages = fltarr(nScans,256,256)
	noisePixels = fltarr(256,256)
	imageStats = fltarr(6)

	noisePixels[*,*] = 1
	directory = ''

	; done with intializations so prompt user for image directory
	READ, directory, PROMPT=$
	'Enter parent directory under which your raw data is stored: '

	directory = directory + '/'
	close, /ALL
	for loopNum = 0,nScans - 1 do begin
		;strtrim,2 removes leading + trailing blanks
		scanNumString = strtrim(string(dirIndex(loopNum)),2)
		filename = directory+scanNumString+'/pdata/1/2dseq'
		matrixSize = FindMatrixSize(directory, scanNumString) ; get size of image
		nFreqEncode = matrixSize[0]/2
		nPhaseEncode = matrixSize[1]
		nImages = FindNumberImages(directory, scanNumString)  ; # images per scan
		dcOffset = FindDCOffset(directory, scanNumString)	  ; DC offset/image
		mapSlope = FindRecoMapSlope(directory, scanNumString)
		mapOffset = FindRecoMapOffset(directory, scanNumString)
		recoImages[loopNum,*,*] = ProcessReco(filename, matrixSize, mapOffset, mapSlope)
		;now determine the average signal intensity (log value) of the center and
		;left corner of the image.
		;imageStats is an array of values holding the following statisitcs:
		;[centerMean, cornerMean, centerVar, cornerVar, centerSdv, cornerSdv]
		imageStats = ImageSegment(recoImages,nFreqEncode, nPhaseEncode, boxPixels)
		centerMean[loopNum] = imageStats[0]
		cornerMean[loopNum] = imageStats[1]
		centerVar[loopNum] = imageStats[2]
		cornerVar[loopNum] = imageStats[3]
		centerSdv[loopNum] = imageStats[4]
		cornerSdv[loopNum] = imageStats[5]
	endfor

	logIntensities = alog(recoImages) ; nImages x 256 x 256 array
	t2Map = fltarr(nFreqencode,nPhaseEncode)
	m0Map = fltarr(nFreqEncode,nPhaseEncode)

	for i = 0,nFreqEncode - 1 do begin
		for j = 0,nPhaseEncode - 1	do begin
			;check that current pixel in EACH (*) image has lower value
			;than center average of each (*) image. LT will return boolean
			;vector. total will sum boolean vector. if each image's pixel is LT
			;the centerMean value it is most likely a noise pixel, so don't do the
			;T2 fit for that pixel. This is a type of edge detection.
			if fix(total(recoImages[*,i,j] LT centerMean[*])) EQ nScans then begin
				noisePixels[i,j] = 0
			endif else begin
				model = linfit(logIntensities[*,i,j],TE)
				t2Map[i,j] = -1/model[1]
				m0Map[i,j] = exp(model[0])
			endelse
		endfor
	endfor

	live_image, noisePixels
	live_image, t2Map
	live_image, m0Map
end


