; This program produces the T2 parameter map of images acquired using
; the bruker MSME sequence.
; DGB 3/12/03

pro t2MapReco
	asciiConstant = 48
	BAD = 0
	GOOD = 1
	maxScans = 20
	rotate90 = 1
	rotate180 = 2
	rotate270 = 3
	rotateTransOnly = 4
	rotateTrans90 = 5
	rotateTrans180 = 6
	rotateTrans270 = 7
	plotNames = ['Signal Intensity: Scaling', 'Signal Intensity: No Scaling']
	sizeIndex = 3 ;# of elements in array is returned by 'size' as 4th element
	dirIndex = [4,5,6,7,8,9,10];,16]		; image directories
	;dirString = ''
	;READ, dirString, PROMPT=$
	;'Enter directory numbers of your scans, i.e., 1,2,3,4,5: '

	; THIS IS NOT DONE!!!!!!!!!!!!!!!!!
	; or i = 0,strlen(dirString) - 1 do begin
	;	strmid(dirString, i + ,
	;	dirIndex = intarr(strlen(dirString))
	;	print, fix(byte(dirString)) - asciiConstant

	;print, dirIndex
	;dirs = [dirIndex]		; image directories
	;print, fix(dirs)
	TE = [.03,.05,.07,.09,.120,.150,.200]	; TE values (T2 weight)
	;TE = [.200,.150,.120,.09,.07,.05,.03]	; TE values (T2 weight) FLIPPED VERSION
	TEtest = [.01, .02, .03, .06, .08, .150, .180]
	;print, TE
	nIndex = size(dirIndex)		; get size of index array
	nScans = nIndex(sizeIndex)	; # of directories to search through
	pixelsNoScale = fltarr(nIndex(sizeIndex))	; set up pixels array dynamically
	pixelsScale = fltarr(nIndex(sizeIndex))
	recoImages = fltarr(nScans,256,256)
	boxWidth = 10
	boxHeight = 10
	;boxPixels = 40	; # of pixels used for averaging box=> segment image
	centerMean = fltarr(nScans)	;hold boxPixels x boxPixels part of image center
	centerMeanImage = fltarr(nScans,256,256)
	cornerMean = fltarr(nScans)	;"				" 		"      image left corner
	centerVar = fltarr(nScans)
	cornerVar = fltarr(nScans)
	centerSdv = fltarr(nScans)
	cornerSdv = fltarr(nScans)
	logIntensities = fltarr(nScans,256,256)
	noNoiseImages = fltarr(nScans,256,256)
	noisePixels = fltarr(256,256)
	imageStats = fltarr(6)
	kernel = intarr(3,3)	; used for smoothing parameter maps
	kernel(*,*) = 1
	threshold = 0.5
	noisePixels[*,*] = 1
	divergePixels = intarr(2,50)
	divergePixels(*,*) = 0
	directory = ''

	; done with intializations so prompt user for image directory
	READ, directory, PROMPT=$
	'Enter parent directory under which your raw data is stored: '

	directory = directory + '/'
	close, /ALL
	for loopNum = 0,nScans - 1 do begin
		;strtrim,2 removes leading + trailing blanks
		scanNumString = strtrim(string(dirIndex(loopNum)),2)
		filename = directory+scanNumString+'/pdata/1/2dseq'
		matrixSize = FindMatrixSize(directory, scanNumString) ; get size of image
		nFreqEncode = matrixSize[0]/2
		nPhaseEncode = matrixSize[1]
		nImages = FindNumberImages(directory, scanNumString)  ; # images per scan
		dcOffset = FindDCOffset(directory, scanNumString)	  ; DC offset/image
		mapSlope = FindRecoMapSlope(directory, scanNumString)
		mapOffset = FindRecoMapOffset(directory, scanNumString)
		recoImages[loopNum,*,*] = ProcessReco(filename, matrixSize, mapOffset, mapSlope)
		;now determine the average signal intensity (log value) of the center and
		;left corner of the image.
		;imageStats is an array of values holding the following statisitcs:
		;[centerMean, cornerMean, centerVar, cornerVar, centerSdv, cornerSdv]
		imageStats = ImageSegment(recoImages,nFreqEncode,nPhaseEncode,boxWidth,boxHeight)
		centerMean[loopNum] = imageStats[0]
		cornerMean[loopNum] = imageStats[1]
		centerVar[loopNum] = imageStats[2]
		cornerVar[loopNum] = imageStats[3]
		centerSdv[loopNum] = imageStats[4]
		cornerSdv[loopNum] = imageStats[5]
	endfor

	logIntensities = alog(recoImages) ; nImages x 256 x 256 array
	t2MapLin = fltarr(nFreqEncode,nPhaseEncode)
	m0MapLin = fltarr(nFreqEncode,nPhaseEncode)
	t2MapNonLin = fltarr(nFreqEncode,nPhaseEncode)
	m0MapNonLin = fltarr(nFreqEncode,nPhaseEncode)
	moMapLinSmooth = fltarr(nFreqEncode, nPhaseEncode)
	sdT2 = fltarr(nFreqEncode, nPhaseEncode)
	sdM0 = fltarr(nFreqEncode, nPhaseEncode)

	;t2Info = CREATE_STRUCT('t2Map',fltarr(nFreqEncode,nPhaseEncode),'m0Map',fltarr(nFreqEncode,nPhaseEncode))
	t2TestImage = fltarr(nFreqEncode, nPhaseEncode)

	; *************************************************************
	; Do LINEAR curve fitting for T2 using the curvefit routine
	; *************************************************************
	for i = 0,nFreqEncode - 1 do begin
		for j = 0,nPhaseEncode - 1	do begin
			;check that current pixel in EACH (*) image has lower value
			;than center average of each (*) image. LT will return boolean
			;vector. total will sum boolean vector. if each image's pixel is LT
			;the centerMean value it is most likely a noise pixel, so don't do the
			;T2 fit for that pixel. This is a type of edge detection.
			if fix(total(recoImages[*,i,j] LT centerMean[*])) EQ nScans then begin
				noisePixels[i,j] = 0
			endif else begin
				model = linfit(TE,logIntensities[*,i,j]) ; TE=>X, log=>Y
				t2MapLin[i,j] = (-1)/model[1]
				m0MapLin[i,j] = exp(model[0])
			endelse
		endfor
	endfor


	;live_image, rotate(t2MapLin, rotateTrans270), TITLE = 't2MapLin'
	;live_image, rotate(m0MapLin, rotateTrans270), TITLE = 'm0MapLin'

	; *************************************************************
	; Do NON_LINEAR curve fitting for T2 using the curvefit routine
	; *************************************************************

	;average value of m0MapLin matrix is used as initial guess
	;of m0 value for non-linear curve fitting routine.
	;When calculating the average value of m0, use only those pixels
	;that are not noisy. The # of pixels not noisy = # of pixels
	;in noisePixels matrix = 1 (or sum of pixels in noisePixels matrix)
	; Remember to take the inverse Log of M0

	m0FitGuess = alog(total(m0MapLin)/total(noisePixels))
	initGuess = fltarr(2)
	A = fltarr(2)
	yfit = fltarr(2)
	; provide initial guess of
	;non-linear function parameters
	initGuess = [95000,0.60]
	A = initGuess
	W = fltarr(N_ELEMENTS(recoImages[*,1,1]))
	W(*) = 1.0
	converge = 1 ; assume lmfit converges
	temp = 0 ; temp looping variable for divergePixels array
	for i = 0,nFreqEncode - 1 do begin
		for j = 0,nPhaseEncode - 1	do begin
			;check that current pixel in EACH (*) image has lower value
			;than center average of each (*) image. LT will return boolean
			;vector. total will sum boolean vector. if each image's pixel is LT
			;the centerMean value it is most likely a noise pixel, so don't do the
			;T2 fit for that pixel. This is a type of edge detection.
			if fix(total(recoImages[*,i,j] LT centerMean[*])) EQ nScans then begin
				noisePixels[i,j] = 0
			endif else begin
				if (i EQ 129 OR i EQ 171) AND (j EQ 158 OR j EQ 157 OR j EQ 126 or j EQ 127) then begin
					goto, nextFor

				endif else begin
					yfit = DBCurvefit(TE,recoImages(*,i,j),W,A,SIGMA,FUNCTION_NAME = 'T2curveFitFunc')
					;result = lmfit(TE,recoImages(*,i,j),A,WEIGHTS = W,CONVERGENCE = converge, $
					;FUNCTION_NAME = 'T2LMFitFunc2Param')

					;lmfit did not converge so reset guess. This skips over current
					; pixel and gives next pixel a chance to converge.
					;if (converge EQ 0) OR (converge EQ -1) then begin
						;A = initGuess
					;endif

					t2MapNonLin[i,j] = A[1]
					m0MapNonLin[i,j] = A[0]
					; determine which pixels are not converging in the fit
					if (yfit[0] LT 0) AND (yfit[1] LT 0) then begin
						noisePixels[i,j] = 6
						;divergePixels[0,temp] = i
						;divergePixels[1,temp] = j
						;temp = temp + 1
						;if temp GT 20 then goto, quitfor
					endif
				endelse

				;sdT2[i,j] = SIGMA[1]
				;sdM0[i,j] = SIGMA[0]
			endelse
			nextFor:
		endfor
	endfor

	live_image, rotate(noisePixels, rotateTrans270), TITLE = 'noisy pixel map'
	live_image, rotate(t2MapNonLin, rotateTrans270), TITLE = 't2MapNonLin'
	live_image, rotate(m0MapNonLin, rotateTrans270), TITLE = 'm0MapNonLin'
	;live_image, rotate(sdT2, rotateTrans270), TITLE = 'sd of T2'
	;live_image, rotate(sdM0, rotateTrans270), TITLE = 'sd of M0'
	;print, divergePixels
	;live_image, m0Map
	;live_image, t2Map
	;m0MapLinSmooth = SmoothMap(m0MapLin, threshold)
	;t2MapLinSmooth = SmoothMap(t2MapLin, threshold)
	m0MapNonLinSmooth = SmoothMap(m0MapNonLin, threshold)
	t2MapNonLinSmooth = SmoothMap(t2MapNonLin, threshold)

	live_image, rotate(t2MapNonLinSmooth, rotateTrans270), TITLE = 't2MapNonLinSmooth'
	live_image, rotate(m0MapNonLinSmooth, rotateTrans270), TITLE = 'm0MapNonLinSmooth'
end