; This program produces the T2 parameter map of images acquired using
; the bruker MSME sequence.
; DGB 3/12/03

pro t2MapRecoTest
	asciiConstant = 48
	BAD = 0
	GOOD = 1
	maxScans = 20
	rotate90 = 1
	rotate180 = 2
	rotate270 = 3
	rotateTransOnly = 4
	rotateTrans90 = 5
	rotateTrans180 = 6
	rotateTrans270 = 7
	plotNames = ['Signal Intensity: Scaling', 'Signal Intensity: No Scaling']
	sizeIndex = 3 ;# of elements in array is returned by 'size' as 4th element
	dirIndex = [4,5,6,7,8,9,10];,16]		; image directories
	;dirString = ''
	;READ, dirString, PROMPT=$
	;'Enter directory numbers of your scans, i.e., 1,2,3,4,5: '

	; THIS IS NOT DONE!!!!!!!!!!!!!!!!!
	; or i = 0,strlen(dirString) - 1 do begin
	;	strmid(dirString, i + ,
	;	dirIndex = intarr(strlen(dirString))
	;	print, fix(byte(dirString)) - asciiConstant


	TE = [.03,.05,.07,.09,.120,.150,.200]	; TE values (T2 weight)
	;TE = [.200,.150,.120,.09,.07,.05,.03]	; TE values (T2 weight) FLIPPED VERSION
	TEtest = [.01, .02, .03, .06, .08, .150, .180]
	;print, TE
	nIndex = size(dirIndex)		; get size of index array
	nScans = nIndex(sizeIndex)	; # of directories to search through
	pixelsNoScale = fltarr(nIndex(sizeIndex))	; set up pixels array dynamically
	pixelsScale = fltarr(nIndex(sizeIndex))
	recoImages = fltarr(256,256,nScans)
	boxWidth = 10
	boxHeight = 10
	centerMean = fltarr(nScans)				;hold boxPixels x boxPixels part of image center
	centerMeanImage = fltarr(256,256,nScans)
	cornerMean = fltarr(nScans)				;"				" 		"      image left corner
	centerVar = fltarr(nScans)
	cornerVar = fltarr(nScans)
	centerSdv = fltarr(nScans)
	cornerSdv = fltarr(nScans)
	logIntensities = fltarr(256,256,nScans) ; cols,rows,# of images
	noNoiseImages = fltarr(256,256,nScans)	; cols, rows, # of images
	noisePixels = fltarr(256,256)
	imageStats = fltarr(6)
	divergePixels = intarr(2,50)
	kernel = intarr(3,3)					; used for smoothing parameter maps

	; **********************************
	; initialize variables (that need it)
	kernel(*,*) = 1
	threshold = 0.5
	noisePixels[*,*] = 1
	divergePixels(*,*) = 0


	; done with intializations so prompt user for image directory
	READ, directory, PROMPT=$
	'Enter parent directory under which your raw data is stored: '

	directory = directory + '/'
	close, /ALL
	for loopNum = 0,nScans - 1 do begin
		;strtrim,2 removes leading + trailing blanks
		scanNumString = strtrim(string(dirIndex(loopNum)),2)
		filename = directory+scanNumString+'/pdata/1/2dseq'
		matrixSize = FindMatrixSize(directory, scanNumString) ; get size of image
		nFreqEncode = matrixSize[0]/2
		nPhaseEncode = matrixSize[1]
		nImages = FindNumberImages(directory, scanNumString)  ; # images per scan
		dcOffset = FindDCOffset(directory, scanNumString)	  ; DC offset/image
		mapSlope = FindRecoMapSlope(directory, scanNumString)
		mapOffset = FindRecoMapOffset(directory, scanNumString)
		recoImages[*,*,loopNum] = ProcessReco(filename, matrixSize, mapOffset, mapSlope)
		;now determine the average signal intensity (log value) of center and
		;left corner of image. imageStats is an array of values holding the following statisitcs:
		;[centerMean, cornerMean, centerVar, cornerVar, centerSdv, cornerSdv]
		imageStats = ImageSegment(recoImages,nFreqEncode,nPhaseEncode,boxWidth,boxHeight)
		centerMean[loopNum] = imageStats[0]
		cornerMean[loopNum] = imageStats[1]
		centerVar[loopNum] = imageStats[2]
		cornerVar[loopNum] = imageStats[3]
		centerSdv[loopNum] = imageStats[4]
		cornerSdv[loopNum] = imageStats[5]
	endfor

	logIntensities = alog(recoImages) ; 256 x 256 x nImages array
	t2MapLin = fltarr(nFreqEncode,nPhaseEncode)
	m0MapLin = fltarr(nFreqEncode,nPhaseEncode)
	t2MapNonLin = fltarr(nFreqEncode,nPhaseEncode)
	m0MapNonLin = fltarr(nFreqEncode,nPhaseEncode)
	pixelVarMap = fltarr(nFreqEncode,nPhaseEncode)
	pixelSTDMap = fltarr(nFreqEncode,nPhaseEncode)
	moMapLinSmooth = fltarr(nFreqEncode, nPhaseEncode)
	sdT2 = fltarr(nFreqEncode, nPhaseEncode)
	sdM0 = fltarr(nFreqEncode, nPhaseEncode)

	;t2Info = CREATE_STRUCT('t2Map',fltarr(nFreqEncode,nPhaseEncode),'m0Map',fltarr(nFreqEncode,nPhaseEncode))
	t2TestImage = fltarr(nFreqEncode, nPhaseEncode)

	; *************************************************************
	; Do LINEAR curve fitting for T2 using the curvefit routine
	; *************************************************************
	for i = 0,nFreqEncode - 1 do begin
		for j = 0,nPhaseEncode - 1	do begin
			;check that current pixel in EACH (*) image has lower value
			;than center average of each (*) image. LT will return boolean
			;vector. total will sum boolean vector. if each image's pixel is LT
			;the centerMean value it is most likely a noise pixel, so don't do the
			;T2 fit for that pixel. This is a type of edge detection.
			if fix(total(recoImages[i,j,*] LT centerMean[*])) EQ nScans then begin
				noisePixels[i,j] = 0
			endif else begin
				model = linfit(TE,logIntensities[i,j,*]) ; TE=>X, log=>Y
				t2MapLin[i,j] = (-1)/model[1]
				m0MapLin[i,j] = exp(model[0])
			endelse
		endfor
	endfor


	;live_image, rotate(t2MapLin, rotateTrans270), TITLE = 't2MapLin'
	;live_image, rotate(m0MapLin, rotateTrans270), TITLE = 'm0MapLin'


	; before we do nolinear curve fitting, the variance of each pixel
	; across images (for all TR's (or TE's) is calculated. We can then
	; decide to disallow the fitting of a particular pixel according to
	; the variance.

	for i = 0,nFreqEncode - 1 do begin
		for j = 0,nPhaseEncode - 1	do begin
			pixelVarMap[i,j] = variance(recoImages[i,j,*])
			pixelSTDMap[i,j] = stddev(recoImages[i,j,*])
		endfor
	endfor

	;live_image, rotate(pixelVarMap, rotateTrans270), TITLE = 'pixel variance map'
	;live_image, rotate(pixelSTDMap, rotateTrans270), TITLE = 'pixel standard deviation map'

	; *************************************************************
	; Do NON_LINEAR curve fitting for T2 using the curvefit routine
	; *************************************************************
	;average value of m0MapLin and t2MapLin matrix is used as initial guess
	;of m0,t2 values for non-linear curve fitting routine.
	;When calculating the average value of m0,t2, use only those pixels
	;that are not noisy. The # of pixels not noisy = # of pixels
	;in noisePixels matrix that are equal to 1 (or sum of pixels in
	;noisePixels matrix)
	m0FitGuess = total(m0MapLin)/total(noisePixels)
	t2FitGuess = total(t2MapLin)/total(noisePixels)
	;print, m0FitGuess
	;print, t2FitGuess
	;break
	A = fltarr(2)		; store parameters of fitted curve
	yfit = fltarr(2)
	A = [m0FitGuess,t2FitGuess]	; provide initial guess of non-linear parameters
	W = fltarr(N_ELEMENTS(recoImages[1,1,*]))
	W(*) = 1.0
	converge = 1 	; assume convergence
	temp = 0 		; temp looping variable for divergePixels array
	nPixelsMapped = 0

	for i = 0,nFreqEncode - 1 do begin
		for j = 0,nPhaseEncode - 1	do begin
			;check that current pixel in EACH (*) image has lower value
			;than center average of each (*) image. LT will return boolean
			;vector. total will sum boolean vector. if each image's pixel is LT
			;the centerMean value it is most likely a noise pixel, so don't do the
			;T2 fit for that pixel. This is a type of edge detection.
			if fix(total(recoImages[i,j,*] LT centerMean[*])) EQ nScans then begin
				noisePixels[i,j] = 0
			endif else begin

					yfit = DBCurvefit(TE,transpose(recoImages[i,j,*]),W,A,SIGMA,FUNCTION_NAME = 'T2curveFitFunc',$
					CONVERGENCE=converge )

					;if curvefit does not converge reset A (guess). If at least one pixel has already
					;been mapped, reset the parameter guess (A) values to the average of the
					;values that have already been mapped. This provides curvefit with a
					;good guess for the next iteration.
					if (converge EQ 0) OR (converge EQ -1) then begin	; do NOT map the current values in A!

						if nPixelsMapped GT 1 then begin
							A[1] = total(t2MapNonLin)/nPixelsMapped ; A[1] = average of t2 map for next iteration
							A[0] = total(m0MapNonLin)/nPixelsMapped	; A[0] = average of m0 map for next iteration
						endif

					endif else begin ;converge EQ 1
						; curvefit converged so we can map the current pixel's t2 and m0 value
						nPixelsMapped = nPixelsMapped + 1
						t2MapNonLin[i,j] = A[1]
						m0MapNonLin[i,j] = A[0]
					endelse
			endelse
		endfor
	endfor

	;live_image, rotate(noisePixels, rotateTrans270), TITLE = 'noisy pixel map'
	live_image, rotate(t2MapNonLin, rotateTrans270), TITLE = 't2MapNonLin'
	live_image, rotate(m0MapNonLin, rotateTrans270), TITLE = 'm0MapNonLin'
	;live_image, rotate(sdT2, rotateTrans270), TITLE = 'sd of T2'
	;live_image, rotate(sdM0, rotateTrans270), TITLE = 'sd of M0'
	;print, divergePixels
	;live_image, m0Map
	;live_image, t2Map
	;m0MapLinSmooth = SmoothMap(m0MapLin, threshold)
	;t2MapLinSmooth = SmoothMap(t2MapLin, threshold)
	m0MapNonLinSmooth = SmoothMap(m0MapNonLin, 0.25)
	t2MapNonLinSmooth1 = SmoothMap(t2MapNonLin, 0.25)
	t2MapNonLinSmooth1Old = t2MapNonLinSmooth1
	; remember that IDL modifies the arguments!!!! this is a bad design
	t2MapNonLinSmooth2 = SmoothMap(t2MapNonLinSmooth1, 0.25)

	;live_image, rotate(t2MapNonLinSmooth1Old, rotateTrans270), TITLE = 't2MapNonLinSmooth1'
	live_image, rotate(t2MapNonLinSmooth2, rotateTrans270), TITLE = 't2MapNonLinSmooth2'
	live_image, rotate(m0MapNonLinSmooth, rotateTrans270), TITLE = 'm0MapNonLinSmooth'
end