; linMap
; routine to map T1 or T2 values using IDL's linear fit
; procedure (linfit)
;
pro LinMap,t2MapLin,m0MapLin,images,nFreq,nPhase,noise,nImages,TE

	logIntensities = alog(images) ; nImages x nFreq x nPhase array
	noiseFloor = indgen(nImages)
	noiseFloor[*] = noise

	for i = 0,nFreq - 1 do begin
		for j = 0,nPhase - 1 do begin
			; compare current pixel in each image to noise threshold.
			; if pixel value is less than noise value, reject the pixel
			; across all images
			goodPixels = where(images[*,i,j] GT noiseFloor[*])
			; fit may be done across all images, or only 2 (at least 2 TE's and 2 images)
			if (goodPixels[0] NE -1) AND (N_ELEMENTS(goodPixels) GT 1) then begin
				model = linfit(TE[goodPixels],logIntensities[goodPixels,i,j]) ; TE=>X, log=>Y
				t2MapLin[i,j] = (-1)/model[1]
				m0MapLin[i,j] = exp(model[0])
			endif
		endfor
	endfor

end
