; This program produces the T2 parameter map of images acquired using
; the bruker MSME sequence.
; DGB 3/12/03

pro t2MapBruker
	asciiConstant = 48
	BAD = 0
	GOOD = 1
	maxScans = 20
	rotate90 = 1
	rotate180 = 2
	rotate270 = 3
	rotateTransOnly = 4
	rotateTrans90 = 5
	rotateTrans180 = 6
	rotateTrans270 = 7
	plotNames = ['Signal Intensity: Scaling', 'Signal Intensity: No Scaling']
	sizeIndex = 3 ;# of elements in array is returned by 'size' as 4th element
	dirIndex = [4,5,6,7,8,9,10];,16]		; image directories
	;dirString = ''
	;READ, dirString, PROMPT=$
	;'Enter directory numbers of your scans, i.e., 1,2,3,4,5: '

	; THIS IS NOT DONE!!!!!!!!!!!!!!!!!
	; or i = 0,strlen(dirString) - 1 do begin
	;	strmid(dirString, i + ,
	;	dirIndex = intarr(strlen(dirString))
	;	print, fix(byte(dirString)) - asciiConstant


	TE = [.03,.05,.07,.09,.120,.150,.200]	; TE values (T2 weight)
	;TE = [.200,.150,.120,.09,.07,.05,.03]	; TE values (T2 weight) FLIPPED VERSION
	TEtest = [.01, .02, .03, .06, .08, .150, .180]
	;print, TE
	nIndex = size(dirIndex)					; get size of index array
	nScans = nIndex(sizeIndex)				; # of directories to search through
	pixelsNoScale = fltarr(nIndex(sizeIndex))	; set up pixels array dynamically
	pixelsScale = fltarr(nIndex(sizeIndex))
	recoImages = fltarr(256,256,nScans)
	boxWidth = 10
	boxHeight = 10
	centerMean = fltarr(nScans)				;hold boxPixels x boxPixels part of image center
	centerMeanImage = fltarr(256,256,nScans)
	cornerMean = fltarr(nScans)				;"				" 		"      image left corner
	centerVar = fltarr(nScans)
	cornerVar = fltarr(nScans)
	centerSdv = fltarr(nScans)
	cornerSdv = fltarr(nScans)
	logIntensities = fltarr(256,256,nScans) ; cols,rows,# of images
	noNoiseImages = fltarr(256,256,nScans)	; cols, rows, # of images
	noisePixels = fltarr(256,256)
	imageStats = fltarr(6)
	divergePixels = intarr(2,50)
	kernel = intarr(3,3)					; used for smoothing parameter maps

	; **********************************
	; initialize variables (that need it)
	kernel(*,*) = 1
	threshold = 0.5
	noisePixels[*,*] = 1
	divergePixels(*,*) = 0


	; done with intializations so prompt user for image directory
	READ, directory, PROMPT=$
	'Enter parent directory under which your raw data is stored: '

	directory = directory + '/'
	close, /ALL
	for loopNum = 0,nScans - 1 do begin
		;strtrim,2 removes leading + trailing blanks
		scanNumString = strtrim(string(dirIndex(loopNum)),2)
		filename = directory+scanNumString+'/pdata/1/2dseq'
		matrixSize = FindMatrixSize(directory, scanNumString) ; get size of image
		nFreqEncode = matrixSize[0]/2
		nPhaseEncode = matrixSize[1]
		nImages = FindNumberImages(directory, scanNumString)  ; # images per scan
		dcOffset = FindDCOffset(directory, scanNumString)	  ; DC offset/image
		mapSlope = FindRecoMapSlope(directory, scanNumString)
		mapOffset = FindRecoMapOffset(directory, scanNumString)
		recoImages[*,*,loopNum] = ProcessReco(filename, matrixSize, mapOffset, mapSlope)
		;now determine the average signal intensity (log value) of center and
		;left corner of image. imageStats is an array of values holding the following statisitcs:
		;[centerMean, cornerMean, centerVar, cornerVar, centerSdv, cornerSdv]
		imageStats = ImageSegment(recoImages,nFreqEncode,nPhaseEncode,boxWidth,boxHeight)
		centerMean[loopNum] = imageStats[0]
		cornerMean[loopNum] = imageStats[1]
		centerVar[loopNum] = imageStats[2]
		cornerVar[loopNum] = imageStats[3]
		centerSdv[loopNum] = imageStats[4]
		cornerSdv[loopNum] = imageStats[5]
	endfor

	logIntensities = alog(recoImages) ; 256 x 256 x nImages array
	t2MapLin = fltarr(nFreqEncode,nPhaseEncode)
	m0MapLin = fltarr(nFreqEncode,nPhaseEncode)
	t2MapNonLin = fltarr(nFreqEncode,nPhaseEncode)
	m0MapNonLin = fltarr(nFreqEncode,nPhaseEncode)
	pixelVarMap = fltarr(nFreqEncode,nPhaseEncode)
	pixelSTDMap = fltarr(nFreqEncode,nPhaseEncode)
	moMapLinSmooth = fltarr(nFreqEncode, nPhaseEncode)
	sdT2 = fltarr(nFreqEncode, nPhaseEncode)
	sdM0 = fltarr(nFreqEncode, nPhaseEncode)

	;t2Info = CREATE_STRUCT('t2Map',fltarr(nFreqEncode,nPhaseEncode),'m0Map',fltarr(nFreqEncode,nPhaseEncode))
	t2TestImage = fltarr(nFreqEncode, nPhaseEncode)

	;*********************************************************************
	; Calculate the noise floor
	;
	CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, images, qValue
	;*********************************************************************

	;print, noiseFloor
	;break
	;*****************************************************************
	;Generate the T2 map using a linear fit procedure
	;
	LinMap,t2MapLin,m0MapLin,images,nFreq,nPhase,noiseFloor,nImages,TE
	;*****************************************************************

	;live_image,t2MapLin,TITLE='t2MapLin'

	t2MapLinNoNeg = t2MapLin
	t2MapLinNoOutLyers = t2MapLin

	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	RemoveNegPixels, t2MapLinNoNeg,nFreq,nPhase
	; *******************************************************************

	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no neg'
	;break
	; *******************************************************************
	; Now get rid of outlying pixels > 0. There shouldn't be too many of
	; these so no need to go through all pixels.
	; outLyerFactor is multiplication factor (mFactor). mFactor*stdev of
	; t2MapLin_2 rows is threshold against which to compare pixels in
	; each row of t2MapLin_2....so pixels with values significantly outside
	; range of other pixels in same row can be removed
	;
	RemoveOutLyers, t2MapLinNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers
	; *******************************************************************

	t2MapLinNoNegNoOutLyers = t2MapLinNoNeg
	RemoveOutLyers, t2MapLinNoNegNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers

	; generate a mask to determine where the noise pixles are/aren't
	; this will be used in calculating the initial t2 guess for
	; the non-linear fit
	noiseIndeces = where(t2MapLinNoNegNoOutLyers EQ 0)
	colsNoise = noiseIndeces MOD nFreq		;modulo 128 gives columns with pixels < 0
											;MOD used here since where is passed entire matrix
	rowsNoise = fix(noiseIndeces / nPhase)	; / 128 then rounded down gives rows with pixels <0
	mask[colsNoise,rowsNoise] = t2MapLinNoNegNoOutLyers[colsNoise,rowsNoise]

	; ***********************************************************************
	; Do NON_LINEAR curve fitting for T2 using the curvefit routine
    ;
	NonLinMap,t2MapNonLin,m0MapNonLin,t2MapLinNoNegNoOutlyers,m0MapLin,mask,$
			  images,nFreq,nPhase,noiseFloor,nImages,TE
	; ***********************************************************************

	;live_image,mask
	;live_image,t2MapLin,TITLE='t2MapLin'
	;live_image,M0MapLin,TITLE='M0MapLin'
	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no negative'
	;live_image,t2MapLinNoOutLyers,TITLE='t2MapLin no outlying'
	live_image,t2MapNonLin,TITLE='t2MapNonLin'
	live_image,t2MapLinNoNegNoOutLyers,TITLE='t2MapLin no neg or outlying'

end