; t2MapZ - for Zhaohui
;
; revision history:
; 	created: 8/?/03
;
;	updated: 8/13/03, add negative pixel removal code
;			 10/22/03: added and refined RemoveOutlyers code and
;					   fixed bugs
;
pro t2MapZ
	noiseFactor = 4 	; multiply this by noise floor value. Higher # = higher noise floor
	outLyerFactor = 4	; This value is multiplied by the standard deviation of the map's rows.
						; This provides a threshold (or maximum) against which all
						; pixels in each row of the map are compared for outlyer removal.
	minOutLyers = 1
	zhaohuiData = 1
	rotateTrans270 = 7	; IDL uses #7 to represent rotation by 270 and transpose for rotate()
	nFreq = 128
	nPhase = 128
	nImages = 32
	noiseFloor = 0
	qValue = 0.001
	TE = [25,50,75,100,125,150,175,200,225,250,275,300,325,350,375,400,425,$
	450,475,500,525,550,575,600,625,650,675,700,725,750,775,800]
	TE = TE *.001 	; convert times to milliseconds
	images = intarr(nFreq,nPhase,nImages)
	outLyerDist = fltarr(1,nPhase)
	leftCol = fltarr(nPhase)
	rightCol = fltarr(nPhase)
	pixelOrder = fltarr(nFreq*nPhase)
	stdImage = fltarr(nPhase)
	tStatsRows = fltarr(2,nPhase) ; tm_test returns [t_stat, significance]
	tStatsCols = fltarr(nFreq,2)
	m0MapLin = fltarr(nFreq,nPhase)
	m0MapNonLin = fltarr(nFreq,nPhase)
	t2MapLin = fltarr(nFreq,nPhase)
	t2MapNonLin = fltarr(nFreq,nPhase)
	mask = fltarr(nFreq,nPhase)
	mask[*,*] = 1

	close, /ALL
	openr, zhaohuiData, 'f:/zhaohui-1-8.sdt', /SWAP_ENDIAN
	readu, zhaohuiData, images	; read unformatted binary data into variable
	close, /ALL

	; rotate and transpose all images so they represent the
	; way they were placed in the magnet, i.e., view from front
	; of magnet
	for is = 0,nImages-1 do begin
		images(*,*,is) = rotate(images(*,*,is),rotateTrans270)
	endfor

	;*********************************************************************
	; Calculate the noise floor
	;
	CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, images, qValue
	;*********************************************************************

	;print, noiseFloor
	;break
	;*****************************************************************
	;Generate the T2 map using a linear fit procedure
	;
	LinMap,t2MapLin,m0MapLin,images,nFreq,nPhase,noiseFloor,nImages,TE
	;*****************************************************************

	;live_image,t2MapLin,TITLE='t2MapLin'

	t2MapLinNoNeg = t2MapLin
	t2MapLinNoOutLyers = t2MapLin

	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	RemoveNegPixels, t2MapLinNoNeg,nFreq,nPhase
	; *******************************************************************

	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no neg'
	;break
	; *******************************************************************
	; Now get rid of outlying pixels > 0. There shouldn't be too many of
	; these so no need to go through all pixels.
	; outLyerFactor is multiplication factor (mFactor). mFactor*stdev of
	; t2MapLin_2 rows is threshold against which to compare pixels in
	; each row of t2MapLin_2....so pixels with values significantly outside
	; range of other pixels in same row can be removed
	;
	RemoveOutLyers, t2MapLinNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers
	; *******************************************************************

	t2MapLinNoNegNoOutLyers = t2MapLinNoNeg
	RemoveOutLyers, t2MapLinNoNegNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers

	; generate a mask to determine where the noise pixles are/aren't
	; this will be used in calculating the initial t2 guess for
	; the non-linear fit
	noiseIndeces = where(t2MapLinNoNegNoOutLyers EQ 0)
	colsNoise = noiseIndeces MOD nFreq		;modulo 128 gives columns with pixels < 0
											;MOD used here since where is passed entire matrix
	rowsNoise = fix(noiseIndeces / nPhase)	; / 128 then rounded down gives rows with pixels <0
	mask[colsNoise,rowsNoise] = t2MapLinNoNegNoOutLyers[colsNoise,rowsNoise]

	; ***********************************************************************
	; Do NON_LINEAR curve fitting for T2 using the curvefit routine
    ;
	NonLinMap,t2MapNonLin,m0MapNonLin,t2MapLinNoNegNoOutlyers,m0MapLin,mask,$
			  images,nFreq,nPhase,noiseFloor,nImages,TE
	; ***********************************************************************

	;live_image,mask
	;live_image,t2MapLin,TITLE='t2MapLin'
	;live_image,M0MapLin,TITLE='M0MapLin'
	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no negative'
	;live_image,t2MapLinNoOutLyers,TITLE='t2MapLin no outlying'
	live_image,t2MapNonLin,TITLE='t2MapNonLin'
	live_image,t2MapLinNoNegNoOutLyers,TITLE='t2MapLin no neg or outlying'
end
