; NonLinMap
;
;	Inputs: t2MapNonLin	=> nonlinear fitted t2 map to be returned
;			m0MapNonLin	=> nonlinear fitted m0 map to be returned
;			t2MapLin	=> average used for initial guess (T2) for curvefit routine
;			m0MapLin 	=> average used for initial guess (m0) for curvefit
;			mask		=> mask (image showing 'good' (i.e., not noisy) pixels only)
;			images		=> actual images acquired on Bruker (or other scanner)
;			nFreq		=> # frequency encode steps = # columns in images
;			nPhase 		=> # phase encode steps = # rows in images
;			noise		=> noise floor
;			Tparam		=> Vector of echo or repeat times (TE or TR) to fit against
;			Type		=> T1 or T2 fitting?
;
;	Outputs:t2MapNonLin	=> yes, this is an output also. nonlinear fitted t2 map
;			m0MapNonLin	=> nonlinear fitted m0 map.

;	average value of m0MapLin and t2MapLin matrix is used as initial guess
;	of m0,t2 values for non-linear curve fitting routine.
;	When calculating the average value of m0,t2, use only those pixels
;	that are not noisy. The # of pixels not noisy = # of pixels
;	in mask matrix that are equal to 1 (or sum of pixels in
;	noisePixels matrix)
;
;	Modification History: Created 8/20/03
;						  8/27/03: add t2MapLin,m0MapLin,mask arguments
;						  11/03/03: removed nImages argument. This can be extrapolated from Tparam.
;
pro NonLinMap,t2MapNonLin,m0MapNonLin,t2MapLin,m0MapLin,mask,images,nFreq,nPhase,noise,Tparam,Type

	A = fltarr(2)				; store parameters of fitted curve
	yfit = fltarr(2)
	nImages = N_ELEMENTS(Tparam)
	W = fltarr(nImages)
	m0FitGuess = total(m0MapLin)/total(mask)
	t2FitGuess = total(t2MapLin)/total(mask)
	noiseFloor = indgen(nImages)
	noiseFloor[*] = noise
	A = [m0FitGuess,t2FitGuess]	; provide initial guess of non-linear parameters
	print, A
	;break
	W(*) = 1.0
	converge = 1 				; assume convergence
	nPixelsMapped = 0
	nNonConverge = 0
	; DEBUG
	tempImage = fltarr(nFreq,nPhase)
	print, size(tempImage)
	tempImage = images(0,*,*)
	tempImage[203,59] = 10
	live_image, tempImage
	break

	if (Type EQ 'T1') then print, 'Doing T1 non-linear fit' else $
	if (Type EQ 'T2') then print, 'Doing T2 non-linear fit' else $
		print, 'Type of non-linear fit not recognized!!!!!!!!!!!!!'
		;return

	for i = 0,nFreq - 1 do begin
		for j = 0,nPhase - 1 do begin
			; compare current pixel in each image to noise threshold.
			; if pixel value is less than noise value, reject the pixel
			; across all images
			goodPixels = where(images[*,i,j] GT noiseFloor[*])

			; fit may be done across all images, or only 3 (at least 3 TE's and 3 images.
			; Curvefit requires at least 3 data points for the fit)

			if (goodPixels[0] NE -1) AND (N_ELEMENTS(goodPixels) GT 2) then begin
				if (Type EQ 'T2') then begin
					yfit = DBCurvefit(Tparam[goodPixels],transpose(images[goodPixels,i,j]),$
					W,A,SIGMA,FUNCTION_NAME = 'T2curveFitFunc',CONVERGENCE=converge )
				endif else begin
					yfit = DBCurvefit(Tparam[goodPixels],images[goodPixels,i,j],$
					W,A,SIGMA,FUNCTION_NAME = 'T1curveFitFunc',CONVERGENCE=converge )
				endelse
				;break

				;if curvefit does not converge reset A (guess). If at least one pixel has already
				;been mapped, reset the parameter guess (A) values to the average of the
				;values that have already been mapped. This provides curvefit with a
				;good guess for the next iteration.
				if (converge EQ 0) OR (converge EQ -1) then begin	; do NOT map the current values in A!

					;DEBUG
					print, goodPixels
					print, i
					print, j
					;print, images(*,i,j)
					;print, images(*,i,j+1) 	;North
					;print, images(*,i-1,j)	;West
					;print, images(*,i+1,j)  ;East
					;print, images(*,i,j-1)	;South
					;print, images(*,i-1,j+1);NorthWest
					;print, images(*,i-1,j-1);SouthWest
					;print, images(*,i+1,j-1);SouthEast
					;print, images(*,i+1,j+1);NorthEast
					;plot, images(*,i,j)
					;oplot, images(*,i,j+1) 	;North
					;oplot, images(*,i-1,j)	;West
					;oplot, images(*,i+1,j)  ;East
					;oplot, images(*,i,j-1)	;South
					;oplot, images(*,i-1,j+1);NorthWest
					;oplot, images(*,i-1,j-1);SouthWest
					;oplot, images(*,i+1,j-1);SouthEast
					;oplot, images(*,i+1,j+1);NorthEast
					images(0,i,j) = 10
					live_image, images(0,*,*)
					break
					if nPixelsMapped GT 1 then begin
						A[1] = total(t2MapNonLin)/nPixelsMapped ; A[1] = average of t2 map for next iteration
						A[0] = total(m0MapNonLin)/nPixelsMapped	; A[0] = average of m0 map for next iteration
					endif

				endif else begin
					; curvefit converged so we can map the current pixel's t2 and m0 value
					nPixelsMapped = nPixelsMapped + 1
					t2MapNonLin[i,j] = A[1]
					m0MapNonLin[i,j] = A[0]
				endelse

			endif
		endfor
	endfor

end





