; T1MapDicom
; this version will read in DICOM image format (for images acquired on scanner1)
; revision history:
; 	created: 10/28/03
;
;	11/03/03: renamed to T1MapDicom from t1Map
function T1MapDicom
	noiseFactor = 1.0 	; multiply this by noise floor value. Higher # = higher noise floor
	outLyerFactor = 5.0	; This value is multiplied by the standard deviation of the map's rows.
						; This provides a threshold (or maximum) against which all
						; pixels in each row of the map are compared for outlyer removal.
	minOutLyers = 1
	rotate180 = 2
	rotateTrans270 = 7	; IDL uses #7 to represent rotation by 270 and transpose for rotate()

    imageTR100TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR100TE10'),rotate180)	; read DICOM formatted image
    imageTR400TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR400TE10'),rotate180)	; read DICOM formatted image
    imageTR800TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR800TE10'),rotate180)	; read DICOM formatted image
    imageTR1200TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR1200TE10'),rotate180)	; read DICOM formatted image
    imageTR1500TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR1500TE10'),rotate180)	; read DICOM formatted image
    imageTR3000TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR3000TE10'),rotate180)	; read DICOM formatted image

	;live_image,imageTR100TE10,TITLE='TR100TE10'
	;live_image,imageTR400TE10,TITLE='TR400TE10'
	;live_image,imageTR800TE10,TITLE='TR800TE10'
	;live_image,imageTR1200TE10,TITLE='TR1200TE10'
	;live_image,imageTR1500TE10,TITLE='TR1500TE10'
	;live_image,imageTR3000TE10,TITLE='TR3000TE10'
	temp = size(imageTR100TE10)
	nFreq = temp(1)
	nPhase = temp(2)
	noiseFloor = 0
	qValue = 0.001
	TR = [3000,1500,1200,800,400,100]
	TE = 10
	nImages = N_ELEMENTS(TR)
	TR = TR *.001 	; convert times to milliseconds
	images = intarr(nImages,nFreq,nPhase)
	outLyerDist = fltarr(1,nPhase)
	leftCol = fltarr(nPhase)
	rightCol = fltarr(nPhase)
	pixelOrder = fltarr(nFreq*nPhase);
	stdImage = fltarr(nPhase)
	tStatsRows = fltarr(2,nPhase) ; tm_test returns [t_stat, significance]
	tStatsCols = fltarr(nFreq,2)
	m0MapLin = fltarr(nFreq,nPhase)
	m0MapNonLin = fltarr(nFreq,nPhase)
	t1MapLin1 = fltarr(nFreq,nPhase)
	t1MapLin2 = fltarr(nFreq,nPhase)
	t1MapNonLin = fltarr(nFreq,nPhase)
	mask = fltarr(nFreq,nPhase)
	mask[*,*] = 1
	alpha = 0.50 	; for Hanning filter

	imagesFiltered = intarr(nImages,nFreq,nPhase)
	images(0,*,*) = imageTR100TE10
	images(1,*,*) = imageTR400TE10
	images(2,*,*) = imageTR800TE10
	images(3,*,*) = imageTR1200TE10
	images(4,*,*) = imageTR1500TE10
	images(4,*,*) = imageTR3000TE10

	;*****************************************************************************
	; Calculate the noise floor
	;
	CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, imageTR400TE10, qValue
	;*****************************************************************************

	; DEBUG
	;print, noiseFloor
	;break

	imagesFiltered(0,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(0,*,*)
	imagesFiltered(1,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(1,*,*)
	imagesFiltered(2,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(2,*,*)
	imagesFiltered(3,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(3,*,*)
	imagesFiltered(4,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(4,*,*)
	imagesFiltered(5,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(5,*,*)

	;*******************************************************************
	;Generate the T1 map using a linear fit procedure
	;
	LinMap,t1MapLin1,m0MapLin,images,nFreq,nPhase,noiseFloor,nImages,TR
	;*******************************************************************

	;***************************************************************************
	;Generate the T1 map using a linear fit procedure
	;
	;LinMap,t1MapLin2,m0MapLin,imagesFiltered,nFreq,nPhase,noiseFloor,nImages,TR
	;***************************************************************************

	;live_image,t1MapLin1,TITLE='t1MapLin'
	;live_image,t1MapLin2,TITLE='t1MapLin after filtered raw images'
	;break
	t1MapLinNoNeg1 = t1MapLin1
	t1MapLinNoOutLyers1 = t1MapLin1
	;t1MapLinNoNeg2 = t1MapLin2
	;t1MapLinNoOutLyers2 = t1MapLin2
	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	;RemoveNegPixels, t1MapLinNoNeg2,nFreq,nPhase
	; *******************************************************************

	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	RemoveNegPixels, t1MapLinNoNeg1,nFreq,nPhase
	; *******************************************************************
	;live_image,t1MapLinNoNeg,TITLE='t1MapLin no neg'
	;break

	t1MapLinNoNegNoOutLyers1 = t1MapLinNoNeg1
	;t1MapLinNoNegNoOutLyers2 = t1MapLinNoNeg2

	; *******************************************************************
	; Now get rid of outlying pixels > 0. There shouldn't be too many of
	; these so no need to go through all pixels.
	; outLyerFactor is multiplication factor (mFactor). mFactor*stdev of
	; t1MapLin_2 rows is threshold against which to compare pixels in
	; each row of t1MapLin_2....so pixels with values significantly outside
	; range of other pixels in same row can be removed
	;
	RemoveOutLyers, t1MapLinNoNegNoOutLyers1,nFreq,nPhase,outLyerFactor,minOutLyers
	;RemoveOutLyers, m0MapLin,nFreq,nPhase,outLyerFactor,minOutLyers
	; *******************************************************************
	;live_image,t1MapLinNoNegNoOutLyers,TITLE='t1MapLin without negative or outlying pixels'
	;break

	; generate a mask to determine where the noise pixles are/aren't
	; this will be used in calculating the initial t1 guess for
	; the non-linear fit
	;noiseIndeces = where(t1MapLinNoNegNoOutLyers1 EQ 0)
	;colsNoise = noiseIndeces MOD nFreq		;modulo 128 gives columns with pixels = 0
											;MOD used here since where is passed entire matrix
	;rowsNoise = fix(noiseIndeces / nPhase)	; / 128 then rounded down gives rows with pixels = 0
	;mask[colsNoise,rowsNoise] = t1MapLinNoNegNoOutLyers1[colsNoise,rowsNoise]
	;live_image, mask
	;break
	;live_image, m0MapLin
	;break
	; ***********************************************************************
	; Do NON_LINEAR curve fitting for t1 using the curvefit routine
    ;
	;NonLinMap,t1MapNonLin,m0MapNonLin,t1MapLinNoNegNoOutlyers,m0MapLin,mask,$
	;		  images,nFreq,nPhase,noiseFloor,TR,'T1'
	; ***********************************************************************

	;live_image,t1MapLin,TITLE='t1MapLin'
	;live_image,M0MapLin,TITLE='M0MapLin'
	;live_image,t1MapLinNoNeg,TITLE='t1MapLin no negative'
	;live_image,t1MapLinNoOutLyers,TITLE='t1MapLin no outlying'
	;live_image,t1MapNonLin,TITLE='t1MapNonLin'
	;live_image,t1MapLinNoNegNoOutLyers,TITLE='t1MapLin no neg or outlying'
	return,t1MapLinNoNegNoOutLyers1
	;live_image,t1MapLinNoNegNoOutLyers1
	;live_image,t1MapLinNoNegNoOutLyers2
	;live_image,m0MapLin
end
