; T2Map
;
; revision history:
; 	created: 10/28/03
;
;  this version will read in DICOM image format (for images acquired on scanner1)
;
;
pro T2Map
	noiseFactor = .5 	; multiply this by noise floor value. Higher # = higher noise floor
	outLyerFactor = 4	; This value is multiplied by the standard deviation of the map's rows.
						; This provides a threshold (or maximum) against which all
						; pixels in each row of the map are compared for outlyer removal.
	minOutLyers = 1
	zhaohuiData = 1
	rotate180 = 2
	rotateTrans270 = 7	; IDL uses #7 to represent rotation by 270 and transpose for rotate()

	imageTE10TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE10TR3'),rotate180)		; read DICOM formatted image
	imageTE50TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE50TR3'),rotate180)		; read DICOM formatted image
	imageTE100TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE100TR3'),rotate180)		; read DICOM formatted image
	imageTE150TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE150TR3'),rotate180)		; read DICOM formatted image
	imageTE250TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE250TR3'),rotate180)		; read DICOM formatted image
    imageTR100TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR100TE10'),rotate180)	; read DICOM formatted image
    imageTR100TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR100TE10'),rotate180)	; read DICOM formatted image
    imageTR100TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR100TE10'),rotate180)	; read DICOM formatted image
    imageTR100TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR100TE10'),rotate180)	; read DICOM formatted image
    imageTR100TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR100TE10'),rotate180)	; read DICOM formatted image
    imageTR100TE10 = rotate(read_dicom('f:/images/brain/mattO/imageTR100TE10'),rotate180)	; read DICOM formatted image

	;live_image,imageTE10TR3
	;break
	temp = size(imageTE10TR3)
	nFreq = temp(1)
	nPhase = temp(2)
	images = intarr(nFreq,nPhase)
	noiseFloor = 0
	qValue = 0.001
	TE = [10,50,100,150,250]
	TR = 3000
	nImages = N_ELEMENTS(TE)
	TE = TE *.001 	; convert times to milliseconds
	images = intarr(nFreq,nPhase,nImages)
	outLyerDist = fltarr(1,nPhase)
	leftCol = fltarr(nPhase)
	rightCol = fltarr(nPhase)
	pixelOrder = fltarr(nFreq*nPhase);
	stdImage = fltarr(nPhase)
	tStatsRows = fltarr(2,nPhase) ; tm_test returns [t_stat, significance]
	tStatsCols = fltarr(nFreq,2)
	m0MapLin = fltarr(nFreq,nPhase)
	m0MapNonLin = fltarr(nFreq,nPhase)
	t2MapLin = fltarr(nFreq,nPhase)
	t2MapNonLin = fltarr(nFreq,nPhase)
	mask = fltarr(nFreq,nPhase)
	mask[*,*] = 1


	;*********************************************************************
	; Calculate the noise floor
	;
	CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, imageTE10TR3, qValue
	;*********************************************************************

	print, noiseFloor
	;break
	;*****************************************************************
	;Generate the T2 map using a linear fit procedure
	;
	LinMap,t2MapLin,m0MapLin,images,nFreq,nPhase,noiseFloor,nImages,TE
	;*****************************************************************

	;live_image,t2MapLin,TITLE='t2MapLin'

	t2MapLinNoNeg = t2MapLin
	t2MapLinNoOutLyers = t2MapLin

	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	RemoveNegPixels, t2MapLinNoNeg,nFreq,nPhase
	; *******************************************************************

	live_image,t2MapLinNoNeg,TITLE='t2MapLin no neg'
	break
	; *******************************************************************
	; Now get rid of outlying pixels > 0. There shouldn't be too many of
	; these so no need to go through all pixels.
	; outLyerFactor is multiplication factor (mFactor). mFactor*stdev of
	; t2MapLin_2 rows is threshold against which to compare pixels in
	; each row of t2MapLin_2....so pixels with values significantly outside
	; range of other pixels in same row can be removed
	;
	RemoveOutLyers, t2MapLinNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers
	; *******************************************************************

	t2MapLinNoNegNoOutLyers = t2MapLinNoNeg
	;RemoveOutLyers, t2MapLinNoNegNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers

	; generate a mask to determine where the noise pixles are/aren't
	; this will be used in calculating the initial t2 guess for
	; the non-linear fit
	;noiseIndeces = where(t2MapLinNoNegNoOutLyers EQ 0)
	colsNoise = noiseIndeces MOD nFreq		;modulo 128 gives columns with pixels < 0
											;MOD used here since where is passed entire matrix
	rowsNoise = fix(noiseIndeces / nPhase)	; / 128 then rounded down gives rows with pixels <0
	mask[colsNoise,rowsNoise] = t2MapLinNoNegNoOutLyers[colsNoise,rowsNoise]

	; ***********************************************************************
	; Do NON_LINEAR curve fitting for T2 using the curvefit routine
    ;
	NonLinMap,t2MapNonLin,m0MapNonLin,t2MapLinNoNegNoOutlyers,m0MapLin,mask,$
			  images,nFreq,nPhase,noiseFloor,nImages,TE
	; ***********************************************************************

	;live_image,mask
	;live_image,t2MapLin,TITLE='t2MapLin'
	;live_image,M0MapLin,TITLE='M0MapLin'
	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no negative'
	;live_image,t2MapLinNoOutLyers,TITLE='t2MapLin no outlying'
	live_image,t2MapNonLin,TITLE='t2MapNonLin'
	live_image,t2MapLinNoNegNoOutLyers,TITLE='t2MapLin no neg or outlying'
end
