; linMap
; routine to map T1 or T2 values using IDL's linear fit
; procedure (linfit)
;
pro TestLinMap,t2MapLin,m0MapLin,images,nFreq,nPhase,noise,nImages,TE

	logIntensities = alog(images) ; nImages x nFreq x nPhase array
	noiseFloor = indgen(nImages)
	noiseFloor[*] = noise
	testImage = images(0,*,*)
	fitChoice = ''
	fitAnother = ''
	fitChoiceMade = 0	; allow multiple choices if first is erroneous, i.e., NOT y or n
	fitAnotherChoiceMade = 0 ; allow multiple choices when choosing to do another voxel fit
	continueFit = 1		; user sets to 0 when he/she is done fitting individual voxels (for map debugging).

	read,fitChoice,prompt = 'Do you want to look at the linear fit for a particular voxel? [y/n]'

	while fitChoiceMade EQ 0 do begin
		case fitChoice of
			'n': fitChoiceMade = 1
			'y': BEGIN
				fitChoiceMade = 1
				print, '********************************************************'
				print, 'Left click on voxel in image to fit, Right click to exit'
				print, '********************************************************'
				tvscl, images(0,*,*)
				cursor, columnFit, rowFit, /DEVICE 	; use /DEVICE to return column # and row #
				while (continueFit EQ 1) do begin

					model = linfit(TE,logIntensities[*,columnFit,rowFit])
					t2 = (-1)/model[1]
					m0 = exp(model[0])
					plot,TE,logIntensities[*,columnFit,rowFit]
					print, 'T2 = '
					print,t2
					print, 'M0 = '
					print, m0
					read,fitAnother,prompt = 'Do you want to fit another voxel? [y/n]'
					while fitAnotherChoiceMade EQ 0 do begin
						case fitAnother of
							'n': BEGIN
								continueFit = 0
								fitAnotherChoiceMade = 1
								 END
							'y': BEGIN
								continueFit = 1
								fitAnotherChoiceMade = 1
								 END
							else: read,fitAnother,prompt = 'Bad Entry, try again! Do you want to fit another voxel? [y/n]'
						endcase ;
					endwhile ;
					fitAnotherChoiceMade = 0 	; reset for error detection in next loop (if user chooses to continue)
				endwhile ;
			   END ; end 'y' case
 			else: read,fitChoice,prompt = 'Bad Entry, try again! [y/n]'
		endcase
	endwhile


	for i = 0,nFreq - 1 do begin
		for j = 0,nPhase - 1 do begin
			; compare current pixel in each image to noise threshold.
			; if pixel value is less than noise value, reject the pixel
			; across all images
			goodPixels = where(images[*,i,j] GT noiseFloor[*])
			; fit may be done across all images, or only 2 (at least 2 TE's and 2 images)
			if (goodPixels[0] NE -1) AND (N_ELEMENTS(goodPixels) GT 1) then begin
				model = linfit(TE[goodPixels],logIntensities[goodPixels,i,j]) ; TE=>X, log=>Y
				t2MapLin[i,j] = (-1)/model[1]
				m0MapLin[i,j] = exp(model[0])
			endif
		endfor
	endfor

end
