; T1MapDicom
; this version will read in DICOM image format (for images acquired on scanner1)
; revision history:
; 	created: 10/28/03
;
;	11/03/03: renamed to T1MapDicom from t1Map
function T1MapDicom, noiseFactor
;	noiseFactor = 1.0 	; multiply this by noise floor value. Higher # = higher noise floor
	showImages = 0
	outLyerFactor = 5.0	; This value is multiplied by the standard deviation of the map's rows.
						; This provides a threshold (or maximum) against which all
						; pixels in each row of the map are compared for outlyer removal.
	minOutLyers = 1
	rotate180 = 2
	rotateTrans270 = 7	; IDL uses #7 to represent rotation by 270 and transpose for rotate()
	t1SlopeSign = 1.0;	; for linmap routine

	imageTR4TE8 = rotate(read_dicom('f:/images/brain/alexia/axTR4TE8'),rotate180)
    imageTR4TE80 = rotate(read_dicom('f:/images/brain/alexia/axTR4TE80'),rotate180)
    imageTR4TE120 = rotate(read_dicom('f:/images/brain/alexia/axTR4TE120'),rotate180)
    imageTR4TE200 = rotate(read_dicom('f:/images/brain/alexia/axTR4TE200'),rotate180)
    imageTR4TE260 = rotate(read_dicom('f:/images/brain/alexia/axTR4TE260'),rotate180)
    imageTR4TE320 = rotate(read_dicom('f:/images/brain/alexia/axTR4TE320'),rotate180)

	if (showImages EQ 1) then begin
		live_image,imageTR4TE8,TITLE='imageTR4TE8'
		live_image,imageTR4TE80,TITLE='imageTR4TE80'
		live_image,imageTR4TE120,TITLE='imageTR4TE120'
		live_image,imageTR4TE200,TITLE='imageTR4TE200'
		live_image,imageTR4TE260,TITLE='imageTR4TE260'
		live_image,imageTR4TE320,TITLE='imageTR4TE320'
	endif

	temp = size(imageTR4TE8)
	nFreq = temp(1)
	nPhase = temp(2)
	noiseFloor = 0
	qValue = 0.001
	;TR = [3.0,1.5,1.2,0.800,0.400,0.100]
	TR = [0.100,0.400,0.800,1.2,1.5];,3.0]		; don't include the TR for image used to estimate m0
	nImages = N_ELEMENTS(TR)
	imagesToFit = intarr(nImages,nFreq,nPhase)
	m0MapLin = fltarr(nFreq,nPhase)			; save the M0 map created via linear fitting
	m0MapNonLin = fltarr(nFreq,nPhase)
	m0Image = fltarr(nFreq,nPhase)			; need an estimate for M0, choose image with longest TR
	t1MapLin1 = fltarr(nFreq,nPhase)
	t1MapLin2 = fltarr(nFreq,nPhase)
	t1MapNonLin = fltarr(nFreq,nPhase)
	mask = fltarr(nFreq,nPhase)
	mask[*,*] = 1
	alpha = 0.50 	; for Hanning filter

	imagesFiltered = intarr(nImages,nFreq,nPhase)
	m0Image = imageTR4TE8

	; the y-axis of the linear fit curve is ln(M0 -Mz(t)). So, we MUST subtract
	;  Mz(t) from M0 before doing the natural log (within Linmap routine)
	imagesToFit(0,*,*) = m0Image - imageTR4TE80
	imagesToFit(1,*,*) = m0Image - imageTR4TE120
	imagesToFit(2,*,*) = m0Image - imageTR4TE200
	imagesToFit(3,*,*) = m0Image - imageTR4TE260
	imagesToFit(4,*,*) = m0Image - imageTR4TE320


	;*****************************************************************************
	; Calculate the noise floor
	;
	CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, imageTR4TE80, qValue
	;*****************************************************************************

	;DEBUG
	;print, noiseFloor
	;break

	;imagesFiltered(0,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(0,*,*)
	;imagesFiltered(1,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(1,*,*)
	;imagesFiltered(2,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(2,*,*)
	;imagesFiltered(3,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(3,*,*)
	;imagesFiltered(4,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(4,*,*)
	;imagesFiltered(5,*,*) = hanning(nFreq,nPhase,ALPHA=alpha) * images(5,*,*)


	;***********************************************************************
	;Generate the T1 map using a linear fit procedure
	;
	LinMap,t1MapLin1,m0MapLin,imagesToFit,nFreq,nPhase,noiseFloor,nImages,TR
	;***********************************************************************

	;print, where(t1MapLin1 LT 0)
	;break
	;***************************************************************************
	;Generate the T1 map using a linear fit procedure
	;
	;LinMap,t1MapLin2,m0MapLin,imagesFiltered,nFreq,nPhase,noiseFloor,nImages,TR
	;***************************************************************************

	;live_image,t1MapLin1,TITLE='t1MapLin'
	;live_image,t1MapLin2,TITLE='t1MapLin after filtered raw images'
	;break
	t1MapLinNoNeg1 = t1MapLin1
	t1MapLinNoOutLyers1 = t1MapLin1
	;t1MapLinNoNeg2 = t1MapLin2
	;t1MapLinNoOutLyers2 = t1MapLin2
	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	;RemoveNegPixels, t1MapLinNoNeg2,nFreq,nPhase
	; *******************************************************************

	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	RemoveNegPixels, t1MapLinNoNeg1,nFreq,nPhase
	; *******************************************************************
	;live_image,t1MapLinNoNeg,TITLE='t1MapLin no neg'
	;break

	t1MapLinNoNegNoOutLyers1 = t1MapLinNoNeg1
	;t1MapLinNoNegNoOutLyers2 = t1MapLinNoNeg2

	; *******************************************************************
	; Now get rid of outlying pixels > 0. There shouldn't be too many of
	; these so no need to go through all pixels.
	; outLyerFactor is multiplication factor (mFactor). mFactor*stdev of
	; t1MapLin_2 rows is threshold against which to compare pixels in
	; each row of t1MapLin_2....so pixels with values significantly outside
	; range of other pixels in same row can be removed
	;
	RemoveOutLyers, t1MapLinNoNegNoOutLyers1,nFreq,nPhase,outLyerFactor,minOutLyers
	;RemoveOutLyers, m0MapLin,nFreq,nPhase,outLyerFactor,minOutLyers
	; *******************************************************************
	;live_image,t1MapLinNoNegNoOutLyers,TITLE='t1MapLin without negative or outlying pixels'
	;break

	; generate a mask to determine where the noise pixles are/aren't
	; this will be used in calculating the initial t1 guess for
	; the non-linear fit
	;noiseIndeces = where(t1MapLinNoNegNoOutLyers1 EQ 0)
	;colsNoise = noiseIndeces MOD nFreq		;modulo 128 gives columns with pixels = 0
    ;									    ;MOD used here since where is passed entire matrix
	;rowsNoise = fix(noiseIndeces / nPhase)	; / 128 then rounded down gives rows with pixels = 0
	;mask[colsNoise,rowsNoise] = t1MapLinNoNegNoOutLyers1[colsNoise,rowsNoise]
	;live_image, mask
	;break
	;live_image, m0MapLin
	;break
	; ***********************************************************************
	; Do NON_LINEAR curve fitting for t1 using the curvefit routine
    ;
	;NonLinMap,t1MapNonLin,m0MapNonLin,t1MapLinNoNegNoOutlyers1,m0MapLin,mask,$
	;		  imagesToFit,nFreq,nPhase,nImages,noiseFloor,TR,'T1'
	; ***********************************************************************

	live_image,m0MapLin, TITLE='M0 Map via Linear routine for T1'
	live_image,t1MapLinNoNegNoOutLyers1, TITLE='T1 Map'
	return,t1MapLinNoNegNoOutLyers1
end
