; T2MapDicom
; this version will read in DICOM image format (for images acquired on scanner1)
; revision history:
; 	created: 10/28/03
;
;	11/03/03: renamed to T2MapDicom from T2Map
function T2MapDicom, noiseFactor
	;noiseFactor = 1.25	; multiply this by noise floor value. Higher # = higher noise floor
	outLyerFactor = 5.0	; This value is multiplied by the standard deviation of the map's rows.
						; This provides a threshold (or maximum) against which all
						; pixels in each row of the map are compared for outlyer removal.

	minOutLyers = 1
	rotate180 = 2
	rotateTrans270 = 7	; IDL uses #7 to represent rotation by 270 and transpose for rotate()
	t2slopeSign = -1.0  ; for linmap routine

	imageTE10TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE10TR3'),rotate180)		; read DICOM formatted image
	imageTE50TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE50TR3'),rotate180)		; read DICOM formatted image
	imageTE100TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE100TR3'),rotate180)		; read DICOM formatted image
	imageTE150TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE150TR3'),rotate180)		; read DICOM formatted image
	imageTE250TR3 = rotate(read_dicom('f:/images/brain/mattO/imageTE250TR3'),rotate180)		; read DICOM formatted image

	live_image,imageTE10TR3,TITLE= 'TE 10 TR 3'
	live_image,imageTE50TR3,TITLE= 'TE 50 TR 3'
	live_image,imageTE100TR3,TITLE= 'TE 100 TR 3'
	live_image,imageTE150TR3,TITLE= 'TE 150 TR 3'
	live_image,imageTE250TR3,TITLE= 'TE 250 TR 3'
	;break
	temp = size(imageTE10TR3)
	nFreq = temp(1)
	nPhase = temp(2)
	;print, temp
	;break
	noiseFloor = 0
	qValue = 0.001
	TE = [0.010,0.050,0.100,0.150,0.250]
	nImages = N_ELEMENTS(TE)
	images = intarr(nImages,nFreq,nPhase)
	m0MapLin = fltarr(nFreq,nPhase)
	m0MapNonLin = fltarr(nFreq,nPhase)
	t2MapLin = fltarr(nFreq,nPhase)
	t2MapNonLin = fltarr(nFreq,nPhase)
	tempImage = fltarr(nFreq,nPhase)
	mask = fltarr(nFreq,nPhase)
	mask[*,*] = 1

	t2Struct = {name, t2Map:fltarr(nFreq,nPhase), m0Map:fltarr(nFreq,nPhase)}
	images(0,*,*) = imageTE10TR3
	images(1,*,*) = imageTE50TR3
	images(2,*,*) = imageTE100TR3
	images(3,*,*) = imageTE150TR3
	images(4,*,*) = imageTE250TR3

	;*****************************************************************************
	; Calculate the noise floor
	;
	CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, imageTE150TR3, qValue
	;*****************************************************************************

	; DEBUG
	;print, noiseFloor
	;break

	;**********************************************************************
	;Test the fitting of particular voxels
	;
	;TestLinMap2,images,nFreq,nPhase,noiseFloor,nImages,TE
	;**********************************************************************

	;*****************************************************************
	;Generate the T2 map using a linear fit procedure
	;
	LinMap,t2MapLin,m0MapLin,images,nFreq,nPhase,noiseFloor,nImages,TE
	;*****************************************************************

	;live_image,t2MapLin,TITLE='t2MapLin'
	;break
	t2MapLinNoNeg = t2MapLin
	t2MapLinNoOutLyers = t2MapLin

	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	RemoveNegPixels, t2MapLinNoNeg,nFreq,nPhase
	; *******************************************************************

	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no neg'
	;break

	t2MapLinNoNegNoOutLyers = t2MapLinNoNeg
	; *******************************************************************
	; Now get rid of outlying pixels > 0. There shouldn't be too many of
	; these so no need to go through all pixels.
	; outLyerFactor is multiplication factor (mFactor). mFactor*stdev of
	; t2MapLin_2 rows is threshold against which to compare pixels in
	; each row of t2MapLin_2....so pixels with values significantly outside
	; range of other pixels in same row can be removed
	;
	RemoveOutLyers, t2MapLinNoNegNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers
	; *******************************************************************
	;live_image,t2MapLinNoNegNoOutLyers,TITLE='t2MapLin without negative or outlying pixels'
	;break

	; generate a mask to determine where the noise pixles are/aren't
	; this will be used in calculating the initial t2 guess for
	; the non-linear fit
	;noiseIndeces = where(t2MapLinNoNegNoOutLyers EQ 0)
	;colsNoise = noiseIndeces MOD nFreq		;modulo 128 gives columns with pixels = 0
											;MOD used here since where is passed entire matrix
	;rowsNoise = fix(noiseIndeces / nPhase)	; / 128 then rounded down gives rows with pixels = 0
	;mask[colsNoise,rowsNoise] = t2MapLinNoNegNoOutLyers[colsNoise,rowsNoise]
	;live_image, mask
	;break
	;live_image, m0MapLin
	;break
	;tempImage = t2MapLinNoNegNoOutlyers
	; ***********************************************************************
	; Do NON_LINEAR curve fitting for T2 using the curvefit routine
    ;
	;NonLinMap,t2MapNonLin,m0MapNonLin,tempImage,m0MapLin,mask,$
	;		  images,nFreq,nPhase,noiseFloor,TE,'T2'
	; ***********************************************************************

	;live_image,t2MapLin,TITLE='t2MapLin'
	;live_image,M0MapLin,TITLE='M0MapLin'
	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no negative'
	;live_image,t2MapLinNoOutLyers,TITLE='t2MapLin no outlying'
	;live_image,t2MapNonLin,TITLE='t2MapNonLin'
	;live_image,t2MapLinNoNegNoOutLyers,TITLE='t2MapLin no neg or outlying'
	;live_image,t2MapLinNoNegNoOutLyers
	;return, t2MapLinNoNegNoOutLyers
	t2Struct.t2Map = t2MapLinNoNegNoOutLyers
	t2Struct.m0Map = m0MapLin
	live_image,m0MapLin, TITLE='M0 Map for T2'
	live_image,t2MapLinNoNegNoOutLyers, TITLE='T2 Map'
	return, t2Struct
	;live_image, t2MapLinNoNegNoOutLyers
	;live_image, m0MapLin
end