; linMap
; routine to map T1 or T2 values using IDL's linear fit
; procedure (linfit)
;
; Inputs: mapLin	=> array to hold the T1 or T2 map (via linear procedure)
;		  m0MapLin	=> array to hold the M0 map (via linear procedure)
;		  images	=> array of images from which the T1 or T2 map is generated
;		  nFreq		=> # of frequency encodings in each image of images array
;		  nPhase 	=> # of phase encodings in each image of images array
;		  noise		=> the noise floor (based on 'center' image in images array)
;		  nImages	=> can you guess?
;		  Tvar		=> array holding either TE or TR values
; Outputs: mapLin	=> T1 or T2 map (via linear fitting) based on images in images array
;		   m0MapLin	=> M0 map (via linear fitting) based on images in images array
;
;	History: created ??
;			 12/2/03: removed TE and renamed to Tvar
;
pro LinMap,mapLin,m0MapLin,images,nFreq,nPhase,noise,nImages,Tvar

	logIntensities = alog(images) ; nImages x nFreq x nPhase array
	noiseFloor = indgen(nImages)
	noiseFloor[*] = noise

	for i = 0,nFreq - 1 do begin
		for j = 0,nPhase - 1 do begin
			; compare current pixel in each image to noise threshold.
			; if pixel value is less than noise value, reject the pixel
			; across all images
			goodVoxels = where(images[*,i,j] GT noiseFloor[*])
			; fit may be done across all images, or only 2 (at least 2 TE's and 2 images)
			if (goodVoxels[0] NE -1) AND (N_ELEMENTS(goodVoxels) GT 1) then begin
				model = linfit(Tvar[goodVoxels],logIntensities[goodVoxels,i,j]) ; TE=>X, log=>Y
				mapLin[i,j] = (-1.0)/model[1]
				m0MapLin[i,j] = exp(model[0])
			endif
		endfor
	endfor

end
