; T2MapDicom
; this version will read in DICOM image format (for images acquired on scanner1)
; revision history:
; 	created: 10/28/03
;
;	11/03/03: renamed to T2MapDicom from T2Map
;	6/21/04: Calculating a noise floor for each image
;
;   9/07/04: changed to pro for testing
;
pro T2MapDicom, noiseFactor


	TE = [0.008,0.080,0.120,0.200,0.260,0.320]
	outLyerFactor = 2.75	; This value is multiplied by the standard deviation of the map's rows.
						; This provides a threshold (or maximum) against which all
						; pixels in each row of the map are compared for pixel outlyer removal.

	noiseQValue = 0.00001
	minOutLyers = 1
	rotate180 = 2
	rotateTrans270 = 7	; IDL uses #7 to represent rotation by 270 and transpose for rotate()
	t2slopeSign = -1.0  ; for linmap routine


	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;;;;; Flag to control display of images
	;;;;; using live_image
	showImages = 1
	;;;;;


	;imageTR4TE8 = rotate(read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_1.dcm'),rotate180)
    ;imageTR4TE80 = rotate(read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_2.dcm'),rotate180)
    ;imageTR4TE120 = rotate(read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_3.dcm'),rotate180)
    ;imageTR4TE200 = rotate(read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_4.dcm'),rotate180)
    ;imageTR4TE260 = rotate(read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_5.dcm'),rotate180)

	imageTR4TE8 = read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_1.dcm')
    imageTR4TE80 = read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_2.dcm')
    imageTR4TE120 = read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_3.dcm')
    imageTR4TE200 = read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_4.dcm')
    imageTR4TE260 = read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_5.dcm')
    ;imageTR4TE320 = rotate(read_dicom('f:/images/brain/alexia_2/alexia_TR3_TE21_1.dcm'),rotate180)

	if (showImages EQ 1) then begin
		live_image,imageTR4TE8,TITLE='imageTR4TE8'
		live_image,imageTR4TE80,TITLE='imageTR4TE80'
		live_image,imageTR4TE120,TITLE='imageTR4TE120'
		live_image,imageTR4TE200,TITLE='imageTR4TE200'
		live_image,imageTR4TE260,TITLE='imageTR4TE260'
		live_image,imageTR4TE320,TITLE='imageTR4TE320'
	endif

	dave

	temp = size(imageTR4TE8)
	nFreq = temp(1)
	nPhase = temp(2)

	nImages = N_ELEMENTS(TE)
	images = intarr(nImages,nFreq,nPhase)
	m0MapLin = fltarr(nFreq,nPhase)
	m0MapNonLin = fltarr(nFreq,nPhase)
	t2MapLin = fltarr(nFreq,nPhase)
	t2MapNonLin = fltarr(nFreq,nPhase)
	tempImage = fltarr(nFreq,nPhase)
	mask = fltarr(nFreq,nPhase)
	mask[*,*] = 1

	;
	; Create a structure to hold the T1 map and M0 map
	;
	t2Struct = {name, t2Map:fltarr(nFreq,nPhase), m0Map:fltarr(nFreq,nPhase)}

	images(0,*,*) = imageTR4TE8
	images(1,*,*) = imageTR4TE80
	images(2,*,*) = imageTR4TE120
	images(3,*,*) = imageTR4TE200
	images(4,*,*) = imageTR4TE260
	images(5,*,*) = imageTR4TE320

	nImages = N_ELEMENTS(images(*,0,0))
	noiseFloor = fltarr(nImages)


	;*******************************************************************************
	; Calculate the noise floor for each image!
	;
	CalcNoiseFloor, noiseFloor, noiseFactor, nFreq, nPhase, images, noiseQValue
	;*******************************************************************************

	;;;; DEBUG
	print, 'noise floor is'
	print, noiseFloor
	DAVE
	;;;;

	;**********************************************************************
	;Test the fitting of particular voxels
	;
	;TestLinMap2,images,nFreq,nPhase,noiseFloor,nImages,TE
	;**********************************************************************

	;*****************************************************************
	;Generate the T2 map using a linear fit procedure
	;
	LinMap,t2MapLin,m0MapLin,images,nFreq,nPhase,noiseFloor,nImages,TE
	;*****************************************************************

	;live_image,t2MapLin,TITLE='t2MapLin'
	;break

	;***********************
	; apply a weighted kernel to the images
	;KernelBlur, t2MapLin

	t2MapLinNoNeg = t2MapLin
	t2MapLinNoOutLyers = t2MapLin



	; *******************************************************************
	; fix up the map somewhat. First, get rid of all mapped pixels < zero
	;
	RemoveNegPixels, t2MapLinNoNeg,nFreq,nPhase
	; *******************************************************************

	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no neg'
	;break

	t2MapLinNoNegNoOutLyers = t2MapLinNoNeg
	; *******************************************************************
	; Now get rid of outlying pixels > 0. There shouldn't be too many of
	; these so no need to go through all pixels.
	; outLyerFactor is multiplication factor (mFactor). mFactor*stdev of
	; t2MapLin_2 rows is threshold against which to compare pixels in
	; each row of t2MapLin_2....so pixels with values significantly outside
	; range of other pixels in same row can be removed
	;
	RemoveOutLyers, t2MapLinNoNegNoOutLyers,nFreq,nPhase,outLyerFactor,minOutLyers
	; ****************************************************************************
	live_image,t2MapLinNoNegNoOutLyers,TITLE='t2MapLin without negative or outlying pixels'
	;break

	; generate a mask to determine where the noise pixles are/aren't
	; this will be used in calculating the initial t2 guess for
	; the non-linear fit
	;noiseIndeces = where(t2MapLinNoNegNoOutLyers EQ 0)
	;colsNoise = noiseIndeces MOD nFreq		;modulo 128 gives columns with pixels = 0
											;MOD used here since where is passed entire matrix
	;rowsNoise = fix(noiseIndeces / nPhase)	; / 128 then rounded down gives rows with pixels = 0
	;mask[colsNoise,rowsNoise] = t2MapLinNoNegNoOutLyers[colsNoise,rowsNoise]
	;live_image, mask
	;break
	;live_image, m0MapLin
	;break
	;tempImage = t2MapLinNoNegNoOutlyers
	; ***********************************************************************
	; Do NON_LINEAR curve fitting for T2 using the curvefit routine
    ;
	;NonLinMap,t2MapNonLin,m0MapNonLin,tempImage,m0MapLin,mask,$
	;		  images,nFreq,nPhase,noiseFloor,TE,'T2'
	; ***********************************************************************

	;live_image,t2MapLin,TITLE='t2MapLin'
	;live_image,M0MapLin,TITLE='M0MapLin'
	;live_image,t2MapLinNoNeg,TITLE='t2MapLin no negative'
	;live_image,t2MapLinNoOutLyers,TITLE='t2MapLin no outlying'
	;live_image,t2MapNonLin,TITLE='t2MapNonLin'
	;live_image,t2MapLinNoNegNoOutLyers,TITLE='t2MapLin no neg or outlying'
	;live_image,t2MapLinNoNegNoOutLyers
	;live_image,m0MapLin, TITLE='M0 Map for T2'
	;live_image,t2MapLinNoNegNoOutLyers, TITLE='T2 Map'

	t2Struct.t2Map = t2MapLinNoNegNoOutLyers
	t2Struct.m0Map = m0MapLin
	;return, t2Struct
end