
; value is an optional argument that lets the caller set the
; initial die value to a value between 1 and 6. UVALUE will simply
; be passed on to the root base of CW_DICE. The TUMBLE keywords let
; the user adjust the tumble count and period
pro cw_dice, parent, value, UVALUE=uvalue, $
	TUMBLE_CNT=tumble_cnt, TUMBLE_PERIOD=tumble_period

; this common block holds the bitmaps and random number
; generator seed
	common cw_dice_blk, seed, faces

; provide defaults for keywords
if not keyword_set(tumble_cnt) then tumble_cnt=10

;guard against a non-sensical request
if (tumble_cnt lt 1) then tumble_cnt=10

;default tumble period in seconds
if not keyword_set(tumble_period) then tumble_period=.05

if (tumble_period lt 0) then tumble_period=0.05

if not keyword_set(uvalue) uvalue=0

; return to caller if error occurs
on_error, 2

; generate the die face bitmaps. The actual code for this is omitted
; here because it doesn't add much to the example, but it can be found
; in the CW_DICE.PRO file.
faces=lonarr(l92)

; use randomu to pick the initial value of the die unless the
; user provided one.
if (n_elements(value) EQ 0) then value = fix(6*randomu(seed) + 1)

; construct a state variable for this instance
state = { value:value, tumble_cnt:FIX(tumble_cnt), $
	tumble_period:tumble_period, remaining:0 }

; create the base widget, passing the UVALUE through for the caller.
; Notice that we also register an event function and GET/SET
; value routines which will be called widget_control on
; our behalf.
base = widget_base(parent, UVALUE=uvalue, $
	event_func='CW_DICE_EVENT',
	func_get_value = 'CW_DICE_GET_VALUE',
	pro_set_value = 'CW_DICE_SET_VALUE')

die = widget_button(base, VALUE= faces(*,*,value-1])
